/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;

import base.ClassificationAction;

import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.conversions.Conversion;

/*
 * These classes are part of the GUI of FunSiP.
 * These classes provide the ability to change, add or remove classificationfeatures
 * from a certain classificationAction. 
 * 
 * Comments in the code are not really necessary, since it is just a gui
 * wrapper around the FunSiP code anyways. 
 * 
 * @author Michiel Van Bel
 *
 */
public class SprClassificationFeaturePanel extends JPanel implements ActionListener{

	public static final long serialVersionUID = 115648l;
	
	private JList actionsList 			= null;
	private JButton addActionButton 	= null;
	private JButton removeActionButton	= null;
	private JButton editActionButton	= null;
	private ClassificationAction ca 	= null;
	
	public SprClassificationFeaturePanel(ClassificationAction ca,Dimension d){
		super();
		//Dimension d = new Dimension(500,200);
		this.setPreferredSize(d);
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		createContent(ca);
	}
	
	public void createContent(ClassificationAction ca){
		this.ca = ca;
		actionsList = new JList();
		actionsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		addActionButton 	= new JButton("Add classification feature");
		removeActionButton  = new JButton("Remove classification feature");
		editActionButton 	= new JButton("Edit classification feature");
		
		addActionButton.addActionListener(this);
		removeActionButton.addActionListener(this);
		editActionButton.addActionListener(this);
		
		JPanel leftPanel = new JPanel(new BorderLayout());
		JPanel rightPanel = new JPanel(new GridLayout(5,1));
		
		JScrollPane scroll = new JScrollPane(actionsList);
		leftPanel.add(scroll,BorderLayout.CENTER);
		
		rightPanel.add(new JPanel().add(addActionButton));
		rightPanel.add(new JPanel().add(new JLabel(" ")));
		rightPanel.add(new JPanel().add(removeActionButton));
		rightPanel.add(new JPanel().add(new JLabel(" ")));
		rightPanel.add(new JPanel().add(editActionButton));
		
		this.add(leftPanel);
		this.add(rightPanel);
		
		//fill up the list with current classificationfeatures
		if(ca!=null&&ca.getClassificationFeatures()!=null){
			List<ClassificationFeature> cfList = ca.getClassificationFeatures();
			actionsList.setListData(createFeaturesArray(cfList));
		}
	}
	
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(addActionButton)){
			SprClassificationFeatureSinglePanel pan = new SprClassificationFeatureSinglePanel(null);
			int ret = JOptionPane.showConfirmDialog(this,pan,"Please provide a classification feature",JOptionPane.OK_CANCEL_OPTION);
			if(ret==JOptionPane.OK_OPTION){
				ClassificationFeature feat = pan.getFeature();
				if(feat!=null){
					//ca shouldn't be null, it should always contain a valid value
					//still, another check couldnt hurt
					if(this.ca !=null){
						this.ca.addClassificationFeature(feat);
						List<ClassificationFeature> cfList = ca.getClassificationFeatures();
						actionsList.setListData(createFeaturesArray(cfList));
					}
				}
			}
		}
		else if(e.getSource().equals(removeActionButton)){
			int selected = actionsList.getSelectedIndex();
			if(selected>=0){
				List<ClassificationFeature> cfList = ca.getClassificationFeatures();
				cfList.remove(selected);
				ca.setClassificationFeatures(cfList);
				actionsList.setListData(createFeaturesArray(cfList));
			}
		}
		else if(e.getSource().equals(editActionButton)){
			int selected = actionsList.getSelectedIndex();
			if(selected>=0){
				ClassificationFeature feat = ca.getClassificationFeatures().get(selected);
				ClassificationFeature newFeat = feat.make_copy();
				SprClassificationFeatureSinglePanel pan = new SprClassificationFeatureSinglePanel(newFeat);
				int ret = JOptionPane.showConfirmDialog(this,pan, "Please edit the classification feature",JOptionPane.OK_CANCEL_OPTION);
				if(ret == JOptionPane.OK_OPTION){
					ClassificationFeature fe = pan.getFeature();
					if(fe!=null){
						if(this.ca!=null){
							this.ca.getClassificationFeatures().set(selected, fe);
							List<ClassificationFeature> cfList = ca.getClassificationFeatures();
							actionsList.setListData(createFeaturesArray(cfList));
						}
					}
				}
			}
		}
	}
	
	private String[] createFeaturesArray(List<ClassificationFeature> cfList){
		String[] res = new String[cfList.size()];
		for(int i=0;i<cfList.size();i++){
			res[i]=cfList.get(i).toString();
		}
		return res;
	}
	
	private class SprClassificationFeatureSinglePanel extends JPanel implements ActionListener{
		
		public static final long serialVersionUID = 1125462558l;
		
		private ClassificationFeature feat = null;
		private JRadioButton topButton = null;
		private JRadioButton downButton = null;
		private JTextField topTextField = null;
		private JTextField downTextField = null;
		private JComboBox comboBox1 = null;
		private JComboBox comboBox2 = null;
		private boolean topSelected = true;
		
		private String[] possibleFeatures = {"","POSITIONAL 1","POSITIONAL 2","POSITIONAL 3",
				"COMPOSITIONAL 1","COMPOSITIONAL 2","COMPOSITIONAL 3",
				"COMPOSITIONAL 4","COMPOSITIONAL 5","COMPOSITIONAL 6",
				"RF","POSITIONAL_GENERAL 1","POSITIONAL_GENERAL 2",
				"POSITIONAL_GENERAL 3","COMPOSITIONAL_GENERAL 1",
				"COMPOSITIONAL_GENERAL 2","COMPOSITIONAL_GENERAL 3",
				"COMPOSITIONAL_GENERAL 4","COMPOSITIONAL_GENERAL 5",
				"COMPOSITIONAL_GENERAL 6","BRANCHPOINT"};
		private boolean[] canDoConversions = {false,false,false,false,
				false,false,false,
				false,false,false,
				false,true,true,
				true,true,
				true,true,
				true,true,
				true,false};
		private String[] possibleConversions = {"","DNA_AA","DNA_RY","DNA_MK","DNA_SW"};
		
		public SprClassificationFeatureSinglePanel(ClassificationFeature feature){
			super();
			this.feat = feature;
			Dimension d = new Dimension(550,250);
			this.setPreferredSize(d);
			this.setMaximumSize(d);
			this.setMinimumSize(d);
			this.setLayout(new GridLayout(2,1));
			
			JPanel fillPanel = new JPanel(new GridLayout(2,1));
			JPanel expPanel = new JPanel(new GridLayout(7,1));
			fillPanel.setBorder(BorderFactory.createEtchedBorder());
			expPanel.setBorder(BorderFactory.createEtchedBorder());
			
			JPanel topPanel = new JPanel();
			JPanel downPanel = new JPanel();
			
			topButton = new JRadioButton(" ");
			downButton = new JRadioButton(" ");
			topButton.addActionListener(this);
			downButton.addActionListener(this);
			ButtonGroup group = new ButtonGroup();
			group.add(topButton);
			group.add(downButton);
			topButton.setSelected(true);
			

			comboBox1 = new JComboBox(possibleFeatures);
			comboBox2 = new JComboBox(possibleConversions);
			topTextField = new JTextField(15);
		
			comboBox1.addActionListener(this);
			comboBox2.setEnabled(false);
			
			downTextField = new JTextField(46);
			
			topPanel.add(topButton);
			topPanel.add(comboBox1);
			topPanel.add(topTextField);
			topPanel.add(comboBox2);
			
			downPanel.add(downButton);
			downPanel.add(downTextField);
			
			JLabel label1 = new JLabel("  a) Select an input way. Either through selection (top) or by direct input (bottom).");
			JLabel label2 = new JLabel("  b) Define the classification feature. The direct input way has access to more classification features.");
			JLabel label3 = new JLabel("  c) Define the parameter(s). Multiple values should all be placed in the same textfield(top).");
			JLabel label4 = new JLabel("  d) If applicable (only features ending in 'general'), define the correct conversion.");
			JLabel label5 = new JLabel(" ");
			JLabel label6 = new JLabel("---> Warning: if incorrect data is inserted (either parameters/features/conversion),");
			JLabel label7 = new JLabel("     the classification feature will not be recognized.");
			expPanel.add(label1);
			expPanel.add(label2);
			expPanel.add(label3);
			expPanel.add(label4);
			expPanel.add(label5);
			expPanel.add(label6);
			expPanel.add(label7);
			
			fillPanel.add(topPanel);
			fillPanel.add(downPanel);
			this.add(fillPanel);
			this.add(expPanel);

			
			if(feature!=null){
				//set the data if feature is not null
				//down
				downTextField.setText(feature.toString());
				
				//up
				String fs = feature.toNoRangeString();
				Conversion c = feature.getConversion();
				String cs = "";
				if(c!=null)
					cs = feature.getConversion().getParamName();
				else
					cs = "";
				int[] up_down = feature.getUpDownRanges();
				comboBox1.setSelectedIndex(getIndexArray(fs,possibleFeatures));
				comboBox2.setSelectedIndex(getIndexArray(cs,possibleConversions));
				if(up_down!=null && up_down[0]!=-1 && up_down[1]!=-1)
					topTextField.setText(up_down[0]+" "+up_down[1]);
				else
					topTextField.setText("");
			}
			else{
				comboBox1.setSelectedIndex(0);
				comboBox2.setSelectedIndex(0);
				topTextField.setText("");
				downTextField.setText("");
			}
			
		}
		
		
		public ClassificationFeature getFeature(){
			if(topSelected){
				String res = "";
				int sel1 = comboBox1.getSelectedIndex();
				if(sel1==-1 || sel1 == 0)  //no classificationfeature selected.
					return null;
				String t1 = (String)comboBox1.getSelectedItem();
				String topString = topTextField.getText();
				String t2 = (String)comboBox2.getSelectedItem();
				res+=t1+" "+topString+" "+t2;
				feat = ClassificationFeatureUtils.parseFeatureFromString(res);
			}
			else{
				String downString = downTextField.getText();
				feat = ClassificationFeatureUtils.parseFeatureFromString(downString);
			}
			return feat;
		}
		
		public void actionPerformed(ActionEvent e){
			if(e.getSource().equals(topButton)||e.getSource().equals(downButton)){
				topSelected = topButton.isSelected();
			}
			if(e.getSource().equals(comboBox1)){
				int index = comboBox1.getSelectedIndex();
				if(index>0 && index <canDoConversions.length){
					boolean b = canDoConversions[index];
					comboBox2.setEnabled(b);
				}	
			}
		}
		
		private int getIndexArray(String s,String[] strings){
			int res = 0;
			if(strings==null||s==null)
				return res;
			for(int i=0;i<strings.length;i++){
				String temp = strings[i];
				if(temp.equals(s)){
					res = i;
					break;
				}
			}
			return res;
		}
		
	}
	
}
