/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;

public class SprOutputPanel extends JPanel implements ActionListener{
	
	public enum OUTPUT{TO_FILE,TO_FRAME_CONSOLE,STANDARD};
	public enum OUTPUT_FILTER{DEBUG,INFO,WARN,ERROR,FATAL};
	
	private OUTPUT output = OUTPUT.TO_FRAME_CONSOLE;
	private OUTPUT_FILTER of = OUTPUT_FILTER.DEBUG;
	
	private JRadioButton debugButton = null;
	private JRadioButton infoButton = null;
	private JRadioButton warnButton = null;
	private JRadioButton errorButton = null;
	private JRadioButton fatalButton = null;
	
	private JRadioButton standardButton = null;
	private JRadioButton toFileButton   = null;
	private JRadioButton toFrameButton  = null;
	
	private JButton chooseFileButton = null;
	private JTextField chooseFileField = null;
	
	public static final long serialVersionUID = 1254486l;
	
	public SprOutputPanel(){
		super();
		
		chooseFileButton = new JButton("Choose output-file");
		chooseFileField = new JTextField(40);
		chooseFileField.setText("./output.txt");
		chooseFileButton.addActionListener(this);
		setFileEnabled(false);		
		
		
		
		standardButton 	= new JRadioButton("Redirect FunSiP output to commandline",false);
		toFileButton	= new JRadioButton("Redirect FunSiP output to file",false);
		toFrameButton	= new JRadioButton("Redirect FunSiP output to gui",true);
		ButtonGroup bgroup = new ButtonGroup();
		bgroup.add(standardButton);
		bgroup.add(toFileButton);
		bgroup.add(toFrameButton);
		standardButton.addActionListener(this);
		toFileButton.addActionListener(this);
		toFrameButton.addActionListener(this);

		
		debugButton 	= new JRadioButton("Debug mode  -- shows { DEBUG, INFO, WARN, ERROR, FATAL }",true);
		infoButton 		= new JRadioButton("Info mode       -- shows { INFO, WARN, ERROR, FATAL }",false);
		warnButton 		= new JRadioButton("Warn mode    -- shows { WARN, ERROR, FATAL }",false);
		errorButton 	= new JRadioButton("Error mode     -- shows { ERROR, FATAL }",false);
		fatalButton 	= new JRadioButton("Fatal mode     -- shows { FATAL }",false);
		ButtonGroup group2 = new ButtonGroup();
		group2.add(debugButton);
		group2.add(infoButton);
		group2.add(warnButton);
		group2.add(errorButton);
		group2.add(fatalButton);
		debugButton.addActionListener(this);
		infoButton.addActionListener(this);
		warnButton.addActionListener(this);
		errorButton.addActionListener(this);
		fatalButton.addActionListener(this);
		
		
		this.setLayout(new BorderLayout());
		
		JPanel topPanel = new JPanel(new GridLayout(6,1));
		topPanel.setBorder(BorderFactory.createEtchedBorder());
		topPanel.add(debugButton);
		topPanel.add(infoButton);
		topPanel.add(warnButton);
		topPanel.add(errorButton);
		topPanel.add(fatalButton);
		topPanel.add(new JLabel(""));
		
		JPanel upPanel = new JPanel(new GridLayout(4,1));
		upPanel.setBorder(BorderFactory.createEtchedBorder());
		JPanel downPanel = new JPanel(new BorderLayout());
		downPanel.setBorder(BorderFactory.createEtchedBorder());
		
		upPanel.add(standardButton);
		upPanel.add(toFileButton);
		upPanel.add(toFrameButton);
		upPanel.add(new JLabel(""));
		downPanel.add(chooseFileButton,BorderLayout.CENTER);
		downPanel.add(chooseFileField,BorderLayout.SOUTH);
		
		this.add(topPanel,BorderLayout.NORTH);
		this.add(upPanel,BorderLayout.CENTER);
		this.add(downPanel,BorderLayout.SOUTH);
	}
	
	public void setFileEnabled(boolean b){
		chooseFileButton.setEnabled(b);
		chooseFileField.setEnabled(b);
	}
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(standardButton)){
			output = OUTPUT.STANDARD;
			setFileEnabled(false);
		}
		else if(e.getSource().equals(toFileButton)){
			output = OUTPUT.TO_FILE;
			setFileEnabled(true);
		}
		else if(e.getSource().equals(toFrameButton)){
			output = OUTPUT.TO_FRAME_CONSOLE;
			setFileEnabled(false);
		}
		else if(e.getSource().equals(debugButton)){
			of = OUTPUT_FILTER.DEBUG;
		}
		else if(e.getSource().equals(infoButton)){
			of = OUTPUT_FILTER.INFO;
		}
		else if(e.getSource().equals(warnButton)){
			of = OUTPUT_FILTER.WARN;
		}
		else if(e.getSource().equals(errorButton)){
			of = OUTPUT_FILTER.ERROR;
		}
		else if(e.getSource().equals(fatalButton)){
			of = OUTPUT_FILTER.FATAL;
		}
		else if(e.getSource().equals(chooseFileButton)){
			JFileChooser chooser = new JFileChooser();
			int ret = chooser.showOpenDialog(this);
			if(ret==JFileChooser.APPROVE_OPTION){
				String s = chooser.getSelectedFile().getPath();
				chooseFileField.setText(s);
			}
		}
	}

	
	public String getOutputFile(){
		return chooseFileField.getText();
	}
	
	/**
	 * @return the output
	 */
	public OUTPUT getOutput() {
		return output;
	}

	/**
	 * @param output the output to set
	 */
	public void setOutput(OUTPUT output) {
		this.output = output;
	}

	/**
	 * @return the of
	 */
	public OUTPUT_FILTER getOf() {
		return of;
	}

	/**
	 * @param of the of to set
	 */
	public void setOf(OUTPUT_FILTER of) {
		this.of = of;
	}

}
