/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import base.ClassificationAction;
import base.FunSiPWorkflow;
import base.FunSiPWorkflow.MULTITHREADING;




public class SprWorkflowPanel extends JPanel implements ActionListener{
	
	public final static long serialVersionUID = 1125487996l;	
	
	//threading
	private JCheckBox threadingCheck = null;
	private JComboBox threadingList = null;
	private JPanel threadingPanel = null;
	//private String threadingCommand = "MULTITHREADING";
	
	//merging
	private JPanel mergeOutputPanel = null;
	private JCheckBox moCheck = null;
	private JTextField moField = null;
//	private String moCommand = "MERGE_PREDICTION_OUTPUT";
	
	//optimizing
	private JPanel optimizerPanel = null;
	private JCheckBox optimizerCheck = null;
	private JTextField optField1 = null;
	private JTextField optField2 = null;
	private JTextField optField3 = null;
//	private String optimizerCommand = "OPTIMIZER";
	
	
	//adding files to all classifications
	private JPanel 		addFilePanel 	= null;
	private JCheckBox 	addFileCheck	= null;
	private JButton		addFileEditButton = null;
	
	
	
	List<AdditionFile> files = new ArrayList<AdditionFile>();
	
	
	private FunSiPWorkflow flow = null;
	
	public SprWorkflowPanel(FunSiPWorkflow work){
		super();
		this.flow = work;
		
		Dimension d = new Dimension(600,200); 
		this.setPreferredSize(d);
		this.setMaximumSize(d);
		this.setMinimumSize(d);
		createContent();
	}
	
	public void setAccordingToCheck(boolean val,JCheckBox check,JPanel content){
		Component[] comps = content.getComponents();
		for(int i=0;i<comps.length;i++){
			if(!comps[i].equals(check)){
				comps[i].setEnabled(val);
			}
		}
	}
	
	private void createContent(){
		this.setLayout(new GridLayout(4,1));
		
		//multithreading
		threadingPanel = new JPanel();
		threadingCheck = new JCheckBox("Multithreading :");
		//threadingCheck.setEnabled(true);
		threadingCheck.setSelected(true);
		threadingCheck.addActionListener(this);
		String[] threadingPossibilities = {"NO_MULTITHREADING","SINGLE_PC_MULTITHREADING","CLUSTER_MULTITHREADING"};
		threadingList = new JComboBox(threadingPossibilities);
		threadingPanel.add(threadingCheck);
		threadingPanel.add(threadingList);
		threadingPanel.setBorder(BorderFactory.createEtchedBorder());
		this.add(threadingPanel);
		
		//merge output
		mergeOutputPanel = new JPanel();
		moCheck = new JCheckBox("Merge output : ");
		moCheck.setSelected(false);
		moCheck.addActionListener(this);
		moField = new JTextField(40);
		mergeOutputPanel.add(moCheck);
		mergeOutputPanel.add(moField);
		mergeOutputPanel.setBorder(BorderFactory.createEtchedBorder());
		this.add(mergeOutputPanel);
		
		//add files
		addFilePanel			= new JPanel();
		addFileCheck 			= new JCheckBox("Evaluate files");
		addFileCheck.setSelected(true);
		addFileCheck.addActionListener(this);
		addFileEditButton = new JButton("Edit files");
		addFileEditButton.addActionListener(this);
		addFilePanel.add(addFileCheck);
		addFilePanel.add(addFileEditButton);
		addFilePanel.setBorder(BorderFactory.createEtchedBorder());
		this.add(addFilePanel);
		
		//optimizer
		optimizerPanel = new JPanel();
		optimizerCheck = new JCheckBox("Optimization : ");
		optimizerCheck.setSelected(true);
		optimizerCheck.addActionListener(this);
		optField1 = new JTextField(5);
		optField2 = new JTextField(5);
		optField3 = new JTextField(5);
		optimizerPanel.add(optimizerCheck);
		optimizerPanel.add(optField1);
		optimizerPanel.add(optField2);
		optimizerPanel.add(optField3);
		optimizerPanel.setBorder(BorderFactory.createEtchedBorder());
		this.add(optimizerPanel);
		
		if(flow!=null){
			//put the right data in the right places according to the configuration files
			
			//multithreading
			MULTITHREADING mt = flow.getThreading();
			int t = getCorrectIndex(mt.toString(),threadingPossibilities);
			threadingList.setSelectedIndex(t);
			
			//merging
			if(!flow.getInputMergeFiles().isEmpty()&&!flow.getOutputMergeFile().isEmpty()){
				List<String> inMo = flow.getInputMergeFiles().get(0);
				String outMo = flow.getOutputMergeFile().get(0);
				String moString = new String();
				for(int i=0;i<inMo.size();i++){
					moString+=inMo.get(i)+" ";
				}
				moString+=outMo;
				moField.setText(moString);
			}
			else{
				moCheck.setSelected(false);
				setAccordingToCheck(moCheck.isSelected(),moCheck,mergeOutputPanel);
			}
			
			//optimization
			if(flow.isOptimizer()){
				int[] opt = flow.getOptimizerData();
				optField1.setText(opt[0]+"");
				optField2.setText(opt[1]+"");
				optField3.setText(opt[2]+"");
			}
			else{
				optimizerCheck.setSelected(false);
				setAccordingToCheck(optimizerCheck.isSelected(),optimizerCheck,optimizerPanel);
			}
		}
	}
	
	private int getCorrectIndex(String s,String[] strings){
		int res = 0;
		for(int i=0;i<strings.length;i++){
			String temp = strings[i];
			if(temp.equals(s)){
				res = i;
				break;
			}
		}
		return res;
	}
	
	public boolean em(JTextField f){
		if(f==null||f.getText().equals(""))
			return true;
		else
			return false;
	}
	
	public void updateWorkflow(){
		if(threadingCheck.isSelected())
			flow.setThreading(threadingList.getSelectedItem().toString());
		if(moCheck.isSelected()&& !moField.getText().equals("")){
			String[] allData = moField.getText().split(" ");
			String output = allData[allData.length-1];
			List<String>input = new ArrayList<String>();
			for(int i=0;i<allData.length-1;i++)
				input.add(allData[i]);
			flow.addInputMergeFiles(input);
			flow.addOutputMergeFile(output);
		}
		if(optimizerCheck.isSelected()&& !em(optField1) &&!em(optField2)&&!em(optField3)){
			try{
				int t1 = Integer.parseInt(optField1.getText());
				int t2 = Integer.parseInt(optField2.getText());
				int t3 = Integer.parseInt(optField3.getText());
				flow.setOptimizer(true);
				flow.setOptimizer_steps(t1, t2, t3);
			}
			catch(Exception exc){}
		}
			
	}
	
	
	/*public String getWorkflowString(){
		StringBuffer res = new StringBuffer();
		res.append("GENERAL_OPTIONS\n");
		if(threadingCheck.isSelected())
			res.append(threadingCommand+"\t\t\t"+threadingList.getSelectedItem()+"\n");
		if(moCheck.isSelected())
			res.append(moCommand+"\t\t\t"+moField.getText()+"\n");
		if(optimizerCheck.isSelected())
			res.append(optimizerCommand+"\t\t\t"+optField1.getText()+" "+optField2.getText()+" "+optField3.getText()+"\n");
		res.append("END\n");
		return res.toString();
	}*/
	
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(threadingCheck)){
			setAccordingToCheck(threadingCheck.isSelected(),threadingCheck,threadingPanel);
		}
		else if(e.getSource().equals(moCheck)){
			setAccordingToCheck(moCheck.isSelected(),moCheck,mergeOutputPanel);
		}
		else if(e.getSource().equals(optimizerCheck)){
			setAccordingToCheck(optimizerCheck.isSelected(),optimizerCheck,optimizerPanel);
		}
		else if(e.getSource().equals(addFileCheck)){
			setAccordingToCheck(addFileCheck.isSelected(),addFileCheck,addFilePanel);
		}
		else if(e.getSource().equals(addFileEditButton)){
			if(flow==null)
				return;
			//first, display the filechoosers 
			int selected = JOptionPane.showConfirmDialog(this,new ChooseAdditionFilePanel(),"Select files",JOptionPane.OK_CANCEL_OPTION);
			//then, act accordingly
			if(selected==JOptionPane.OK_OPTION){
				List<String> nonMerge = new ArrayList<String>();
				List<String> merge = new ArrayList<String>();
				for(int i=0;i<files.size();i++){
					AdditionFile af = files.get(i);
					if(af.mergeFile)
						merge.add(af.filePath);
					else
						nonMerge.add(af.filePath);
				}
				
				//non-merges
				for(int i=0;i<nonMerge.size();i++){
					String filePath = nonMerge.get(i);
					List<ClassificationAction> actions = flow.getClassificationActions();
					for(int j=0;j<actions.size();j++){
						String[] temp = actions.get(j).getTestfiles();
						String[] newTemp = new String[temp.length+1];
						System.arraycopy(temp, 0, newTemp, 0, temp.length);
						newTemp[temp.length]=filePath;
						actions.get(j).setTestfiles(newTemp);
					}
				}
				
				//merges
				File[] files = new File[merge.size()];
				for(int i=0;i<merge.size();i++){
					files[i] = new File(merge.get(i));
				}
				flow.setExplicitFiles(files);
				
			}
			else{
				files.clear();
			}
		}
	}
	
	private class AdditionFile {
		
		public AdditionFile(String s){
			filePath  = s;
		}
		public AdditionFile(String s, boolean b){
			filePath = s;
			mergeFile = b;
		}
		public String filePath = "";
		public boolean mergeFile = true;
		
		public String toString(){
			return filePath;
		}
	}
	
	private class ChooseAdditionFilePanel extends JPanel implements ActionListener, ListSelectionListener{
		
		public static final long serialVersionUID = 123456789465l;
		
		private JButton addButton = null;
		private JButton removeButton = null;
		private JCheckBox mergeCheck = null;
		private JList filesList = null;
		
		public ChooseAdditionFilePanel(){
			super();
			this.setLayout(new BorderLayout());
			Dimension d = new Dimension(400,300);
			this.setPreferredSize(d);
			this.setMinimumSize(d);
			this.setMaximumSize(d);
			
			filesList = new JList();
			filesList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
			filesList.addListSelectionListener(this);
			
			addButton = new JButton("Add file");
			removeButton = new JButton("Remove file");
			mergeCheck  = new JCheckBox("Merge outputfiles");
			
			addButton.addActionListener(this);
			removeButton.addActionListener(this);
			mergeCheck.addActionListener(this);
			
			//listener voor
			
			//JPanel lPanel = new JPanel();
			//lPanel.setBorder(BorderFactory.createEtchedBorder());
			//lPanel.add(new JScrollPane(filesList));
			//this.add(lPanel,BorderLayout.CENTER);
			this.add(new JScrollPane(filesList),BorderLayout.CENTER);
			
			JPanel checkPanel = new JPanel(new GridLayout(2,1));
			checkPanel.add(mergeCheck);
			checkPanel.add(new JLabel(""));
			checkPanel.setBorder(BorderFactory.createEtchedBorder());
			
			JPanel buttonPanel = new JPanel(new GridLayout(4,1));
			buttonPanel.add(addButton);
			buttonPanel.add(new JLabel(""));
			buttonPanel.add(removeButton);
			buttonPanel.add(new JLabel(""));
			buttonPanel.setBorder(BorderFactory.createEtchedBorder());
			
			JPanel dPanel = new JPanel(new BorderLayout());
			dPanel.add(checkPanel,BorderLayout.NORTH);
			dPanel.add(buttonPanel,BorderLayout.SOUTH);
			this.add(dPanel,BorderLayout.EAST);
			
			
		}
		
		public void valueChanged(ListSelectionEvent e){
			if(filesList.getSelectedIndex()>=0){
				AdditionFile af = (AdditionFile)filesList.getSelectedValue();
				mergeCheck.setSelected(af.mergeFile);
			}
		}
		
		public void actionPerformed(ActionEvent e){
			if(e.getSource().equals(addButton)){
				JFileChooser chooser = new JFileChooser("./");
				int returnVal = chooser.showOpenDialog(this);
				if(returnVal==JFileChooser.APPROVE_OPTION){
					String file = chooser.getSelectedFile().getAbsolutePath();
					AdditionFile af = new AdditionFile(file);
					files.add(af);
				}
			}
			else if(e.getSource().equals(removeButton)){
				if(filesList.getSelectedIndex()>=0){
					AdditionFile af = (AdditionFile)filesList.getSelectedValue();
					files.remove(af);
				}
			}
			
			else if(e.getSource().equals(mergeCheck)){
				if(filesList.getSelectedIndex()>=0){
					AdditionFile af = files.get(filesList.getSelectedIndex());
					af.mergeFile = mergeCheck.isSelected();
				}
			}
			
			updateDisplayList();
		}
		
		public void updateDisplayList(){
			AdditionFile[] afs = new AdditionFile[files.size()];
			for(int i=0;i<files.size();i++)
				afs[i]=files.get(i);
			filesList.setListData(afs);
		}
		
	}
	
	
	

}
