/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package base.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.EventQueue;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.event.WindowListener;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import main.FunSiPGui;
import main.FunSiPJar;

import org.apache.log4j.Logger;
import org.apache.log4j.varia.NullAppender;

import base.ClassificationAction;
import base.FunSiPWorkflow;
import base.ParameterFileParser;


import util.featureSelection.FeatureSelectionStub;
import util.io.ModelDataParser;
import util.io.ModelDataParser.ModelData;

public class WizardFramework extends JFrame implements DocumentListener, WindowListener, WindowFocusListener, ActionListener,ListSelectionListener{
	
	
	public static final long serialVersionUID = 125448654l;
		
	public enum WIZARD_STATE {BUILD_OR_USE,SPECIES_USE,SPECIES_BUILD,ACTIONS_USE,ACTIONS_BUILD,FINAL_USE,FINAL_BUILD;
	
		public static WIZARD_STATE previous(WIZARD_STATE s){
			WIZARD_STATE[] states = WIZARD_STATE.values();
			int index = getIndex(states,s);
			int newIndex = index-2;
			if(newIndex<0)
				newIndex = 0;
			return states[newIndex];
		}
		public static WIZARD_STATE next(WIZARD_STATE s){
			WIZARD_STATE[] states = WIZARD_STATE.values();
			int index = getIndex(states,s);
			int newIndex = index+2;
			if(newIndex>=states.length)
				newIndex = states.length-1;
			return states[newIndex];
		}
	
		public static int getIndex(WIZARD_STATE[] states, WIZARD_STATE state){
			int result = 0;
			for(int i=0;i<states.length;i++){
				WIZARD_STATE ws = states[i];
				if(ws==state)
					result = i;
			}
			return result;
		}
	};
	
	private WIZARD_STATE state = WIZARD_STATE.BUILD_OR_USE;
	
	private List<ModelData> modelData = null;
	private JPanel mainPanel = null;
	private String finalConfigurationFileName = null;
	private FunSiPGui fspParent = null;
	
	/*
	 * Stuff for the buttonpanel
	 */
	private JPanel buttonPanel 		= null;
	private JButton previousButton	= null;
	private JButton nextButton		= null;
	private JButton cancelButton 	= null;
	private JButton finishButton	= null;
	
	/*
	 * stuff for the build_or_use state panel
	 */
	private JPanel buildOrUsePanel			= null;
	private JRadioButton buildRadioButton	= null;
	private JRadioButton useRadioButton		= null;
	//private JRadioButton optimizeRadioButton= null;
	
	/*
	 * stuff for the species use selection state panel
	 */
	private String[] speciesUse				= null;
	private JPanel speciesUsePanel			= null;
	private JList speciesUseList			= null; 
	private String selectedUseSpecies		= null;
	
	/*
	 * stuff for the actions use selection state panel
	 */
	private String[] actionsUse				= null;
	private JPanel actionsUsePanel			= null;
	private JList actionsUseList			= null;
	private String[] selectedUseAction		= null;
	
	/*
	 * Stuff for the final use state panel
	 */
	private JPanel finalUsePanel			= null;
	private JCheckBox finalUseMergeCheck	= null;
	private JList finalUseFileList			= null;
	private JButton finalUseAddFileButton	= null;
	private JButton finalUseRemoveFileButton= null;
	private String[] finalUseFiles			= null;
	private JTextField finalUseConfigFileNameField	= null;
	
	
	/*
	 * Stuff for the species build state panel
	 */
	private JPanel speciesBuildPanel				= null;
	private JTextField speciesNameBuildField		= null;
	private JTextField speciesPosTrainBuildField 	= null;
	private JTextField speciesNegTrainBuildField 	= null;
	private JTextField locationTrainBuildField		= null;
	private JButton speciesPosTrainBuildButton		= null;
	private JButton speciesNegTrainBuildButton		= null;
	private boolean speciesBuildComplete			= false;
	private int trainLocation						= 0;
	 
	
	/*
	 * Stuff for the actions build state panel 
	 */
	private JPanel actionsBuildPanel 					= null;
	private JCheckBox actionsBuildOptimization			= null;
	private JTextField actionsBuildOptField1			= null;
	private JTextField actionsBuildOptField2 			= null;
	private JTextField actionsBuildOptField3			= null;
	private JTextField actionsBuildName					= null;
	private JTextField actionsBuildClassifName			= null;
	private JButton editClassificationActions			= null;
	private ClassificationAction buildClassificationAction 	= null;
	private boolean actionsBuildComplete				= false;
	
	
	/*
	 * stuff for the finish build state panel
	 */
	private JPanel finishBuildPanel						= null;
	private JCheckBox createEvaluateFile				= null;
	private JCheckBox applyFeatureSelection				= null;
	private JCheckBox posAmountTrainingCheck			= null;
	private JCheckBox negAmountTrainingCheck			= null;
	private JTextField posAmountTrainingField			= null;
	private JTextField negAmountTrainingField			= null;
	private JTextField finishBuildConfigNameField		= null;
	private JCheckBox addToModelFileCheck				= null;
	private int posAmountTraining						= 0;
	private int negAmountTraining						= 0;
	
	
	
	public WizardFramework(FunSiPGui fsg,JFrame parent){
		super("FunSiP wizard");
		this.fspParent = fsg;
		setDimensions(new Dimension(500,400),this);
		
		//load data from the models.txt configuration file
		try{
			ModelDataParser mdParser = new ModelDataParser();
			List<ModelData> md = mdParser.parseModelFile("/config/models.txt");
			extractModels(md);
		}
		catch(Exception exc){
			String message = "An error occured while building the wizard (locating models) : "+exc;
			JOptionPane.showMessageDialog(this,message, "Unable to finish wizard", JOptionPane.ERROR_MESSAGE);
			this.dispose();
		}
		
		createPanels();
		mainPanel = new JPanel(new BorderLayout());
		setMainPanel();
		setButtons();
		
		this.setVisible(true);
		this.setResizable(false);
		this.setLocationRelativeTo(parent);
		this.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);
		this.addWindowFocusListener(this);
		this.setAlwaysOnTop(true);
		this.pack();
	}

	
	public void extractModels(List<ModelData> mdList){
		Set<String> species_set = new HashSet<String>();
		for(int i=0;i<mdList.size();i++)
			species_set.add(mdList.get(i).getSpecies());
		List<String> species_list = new ArrayList<String>();
		species_list.addAll(species_set);
		String[] species = new String[species_set.size()];
		for(int i=0;i<species.length;i++)
			species[i] = species_list.get(i);
		
		this.modelData = mdList;
		this.speciesUse = species;
	}
	
	
	public String[] getPossibleActions(String species){
		if(modelData==null)
			return null;
		List<String> dat = new ArrayList<String>();
		for(int i=0;i<modelData.size();i++){
			ModelData md = modelData.get(i);
			if(md.getSpecies().equals(species)){
				dat.add(md.getPurpose());
			}
		}
		String[] p = new String[dat.size()];
		for(int i=0;i<dat.size();i++)
			p[i]= dat.get(i);
		return p;
	}
	
	public String getModelLocation(String species,String action){
		if(modelData==null)
			return null;
		for(int i=0;i<modelData.size();i++){
			ModelData md = modelData.get(i);
			if(md.getSpecies().equals(species)&&md.getPurpose().equals(action))
				return md.getLocations();
		}
		return null;
	}
	
	
	private void createPanels(){
		Logger tempLogger = Logger.getLogger("tLog");
		tempLogger.addAppender(new NullAppender());
		FunSiPWorkflow fspw = new FunSiPWorkflow(tempLogger,"");
		buildClassificationAction = new ClassificationAction(tempLogger,fspw);
		
		//button panel stuff
		buttonPanel = new JPanel();
		setDimensions(new Dimension(500,50),buttonPanel);
		buttonPanel.setBorder(BorderFactory.createEtchedBorder());
		
		previousButton 		= new JButton("Previous");
		nextButton 			= new JButton("Next");
		finishButton		= new JButton("Finish");
		cancelButton		= new JButton("Cancel");
		
		previousButton.addActionListener(this);
		nextButton.addActionListener(this);
		finishButton.addActionListener(this);
		cancelButton.addActionListener(this);
		
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(previousButton);
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(nextButton);
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(finishButton);
		buttonPanel.add(new JLabel(" "));
		buttonPanel.add(cancelButton);
		buttonPanel.add(new JLabel(" "));
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
		//build or use panel
		buildOrUsePanel 		= new JPanel(new GridLayout(2,1));
		setDimensions(new Dimension(500,200),buildOrUsePanel);
		buildOrUsePanel.setBorder(BorderFactory.createEtchedBorder());
		useRadioButton			= new JRadioButton("Use (an) existing classification model(s)",true);
		buildRadioButton		= new JRadioButton("Build new classification model",false);
		//optimizeRadioButton		= new JRadioButton("Optimize classification features",false);
		
		//buildRadioButton.setEnabled(false);
		ButtonGroup group 		= new ButtonGroup();
		group.add(useRadioButton);
		group.add(buildRadioButton); //actionlisteners are not necessary, state changes are invoked by buttonpanel
	//	group.add(optimizeRadioButton);
				
		JLabel generalBuildOrUseLabel = new JLabel("<html>Please define the purpose of the program:" +
				"You can either chose to use an existing classification model (such as" +
				"for donor/acceptor recognition, start site prediction,...) for some organisms," +
				"or you can chose to build your own classification model (e.g. for " +
				"newly sequenced organisms.</html>");
		
		JPanel buildOrUseTopPanel = new JPanel(new BorderLayout());
		JPanel buildOrUseDownPanel = new JPanel(new GridLayout(5,1));
		
		buildOrUseTopPanel.add(generalBuildOrUseLabel,BorderLayout.CENTER);
		buildOrUseDownPanel.add(new JLabel(" "));
		buildOrUseDownPanel.add(useRadioButton);
		buildOrUseDownPanel.add(new JLabel(" "));
		buildOrUseDownPanel.add(buildRadioButton);
		buildOrUseDownPanel.add(new JLabel(" "));
		//buildOrUseDownPanel.add(optimizeRadioButton);
		//buildOrUseDownPanel.add(new JLabel(" "));
		
		buildOrUsePanel.add(buildOrUseTopPanel);
		buildOrUsePanel.add(buildOrUseDownPanel);
				
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//species Use panel
		speciesUsePanel = new JPanel(new GridLayout(0,1));
		speciesUsePanel.setBorder(BorderFactory.createEtchedBorder());
		setDimensions(new Dimension(500,200),speciesUsePanel);
		speciesUseList 	= new JList(speciesUse);
		speciesUseList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		speciesUseList.addListSelectionListener(this);
		JLabel speciesUseLabel = new JLabel("<HTML> Please define the organism for which you want" +
				" to evaluate certain functional sites. Only one organism can be selected at a time." +
				" Following to this selection, the actions that are available will be displayed.</HTML>");
		JPanel speciesUsePanel1 = new JPanel(new BorderLayout());
		JPanel speciesUsePanel2 = new JPanel(new BorderLayout());
		speciesUsePanel1.add(speciesUseLabel,BorderLayout.CENTER);
		speciesUsePanel2.add(new JScrollPane(speciesUseList));
		speciesUsePanel.add(speciesUsePanel1);
		speciesUsePanel.add(speciesUsePanel2);
		
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//actions Use panel
		actionsUsePanel = new JPanel(new GridLayout(0,1));
		actionsUsePanel.setBorder(BorderFactory.createEtchedBorder());
		setDimensions(new Dimension(500,200),actionsUsePanel);
		actionsUse = getPossibleActions(selectedUseSpecies);
		actionsUseList = new JList(actionsUse);
		//actionsUseList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		actionsUseList.addListSelectionListener(this);
		JLabel actionsUseLabel = new JLabel("<HTML> Please define the kind of prediction that " +
				"needs to be done. Multiple types of predictions can be selected at once. " +
				"It is possible in this case to instrument the program to merge the different" +
				" outputs.</HTML>");
		actionsUsePanel.add(actionsUseLabel);
		actionsUsePanel.add(new JScrollPane(actionsUseList));
		
		
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//finish Use panel
		finalUsePanel = new JPanel(new BorderLayout());
		finalUsePanel.setBorder(BorderFactory.createEtchedBorder());
		setDimensions(new Dimension(500,200),finalUsePanel);
		
		JPanel finalUseSeqFilePanel = new JPanel(new BorderLayout());
		setDimensions(new Dimension(480,150),finalUseSeqFilePanel);
		JPanel finalUseSeqFileButtonPanel = new JPanel();
		finalUseFileList = new JList();
		finalUseFileList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		finalUseAddFileButton = new JButton("Add sequence file");
		finalUseRemoveFileButton = new JButton("Remove sequence file");
		finalUseMergeCheck = new JCheckBox("Merge output");
		finalUseAddFileButton.addActionListener(this);
		finalUseRemoveFileButton.addActionListener(this);
		finalUseSeqFileButtonPanel.add(finalUseAddFileButton);
		finalUseSeqFileButtonPanel.add(finalUseRemoveFileButton);
		finalUseSeqFileButtonPanel.add(new JLabel(" "));
		finalUseSeqFileButtonPanel.add(finalUseMergeCheck);
		JLabel finalUseFileLabel = new JLabel("<HTML>Please provide the sequence fasta files" +
				" from which the functional sites should be examined.</HTML>");
		JScrollPane finalUseFileScroll = new JScrollPane(finalUseFileList);
		setDimensions(new Dimension(400,100),finalUseFileScroll);
		finalUseSeqFilePanel.add(finalUseFileLabel,BorderLayout.SOUTH);
		finalUseSeqFilePanel.add(finalUseFileScroll,BorderLayout.CENTER);
		finalUseSeqFilePanel.add(finalUseSeqFileButtonPanel,BorderLayout.SOUTH);
		finalUseSeqFilePanel.setBorder(BorderFactory.createTitledBorder("Fasta file selection"));
		
		JPanel finalUseConfigFilePanel = new JPanel(new GridLayout(0,1));
		JLabel finalUseConfigFileLabel = new JLabel("<HTML>Please provide a name for the configuration file. </HTML>");
		finalUseConfigFileNameField = new JTextField(30);
		finalUseConfigFilePanel.add(finalUseConfigFileLabel);
		finalUseConfigFilePanel.add(finalUseConfigFileNameField);
		
		finalUsePanel.add(finalUseSeqFilePanel,BorderLayout.CENTER);
		finalUsePanel.add(finalUseConfigFilePanel,BorderLayout.SOUTH);
		
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//species build panel
		speciesBuildPanel = new JPanel(new GridLayout(5,1));
		JPanel snbf1Panel	= new JPanel(new BorderLayout());
		JPanel snbf2Panel	= new JPanel(new GridLayout(1,3));
		JPanel snbf3Panel	= new JPanel(new GridLayout(1,3));
		JPanel snbf4Panel	= new JPanel(new GridLayout(1,3));
		JPanel snbf5Panel	= new JPanel(new GridLayout(1,3));
		
		JLabel speciesNameBuildExpLabel = new JLabel("<html>Please provide the necessary information " +
				"concerning the organism for which the classification model " +
				"should be build. This includes the name of the species and the locations of the files " +
				"with the positive and negative training data.</html>");
		snbf1Panel.add(speciesNameBuildExpLabel,BorderLayout.CENTER);
	
				
		JLabel speciesNameBuildLabel = new JLabel("Name species :");
		speciesNameBuildField = new JTextField(18);
		JLabel speciesNameBuildTempLabel = new JLabel(" ");
		JPanel snbf21Panel = new JPanel(/*new BorderLayout()*/); snbf21Panel.add(speciesNameBuildLabel/*,BorderLayout.CENTER*/);
		JPanel snbf22Panel = new JPanel(); snbf22Panel.add(speciesNameBuildField);
		JPanel snbf23Panel = new JPanel(); snbf23Panel.add(speciesNameBuildTempLabel);
		snbf2Panel.add(snbf21Panel);
		snbf2Panel.add(snbf22Panel);
		snbf2Panel.add(snbf23Panel);
		snbf2Panel.setBorder(BorderFactory.createEtchedBorder());
		
		JLabel speciesNameBuildPosLabel = new JLabel("Positive training file :");
		speciesPosTrainBuildField  = new JTextField(18);
		speciesPosTrainBuildButton = new JButton("Search file");
		JPanel snbf31Panel = new JPanel(/*new BorderLayout()*/); snbf31Panel.add(speciesNameBuildPosLabel/*,BorderLayout.CENTER*/);
		JPanel snbf32Panel = new JPanel(); snbf32Panel.add(speciesPosTrainBuildField);
		JPanel snbf33Panel = new JPanel(); snbf33Panel.add(speciesPosTrainBuildButton);
		snbf3Panel.add(snbf31Panel);
		snbf3Panel.add(snbf32Panel);
		snbf3Panel.add(snbf33Panel);
		snbf3Panel.setBorder(BorderFactory.createEtchedBorder());
		
		JLabel speciesNameBuildNegLabel = new JLabel("Negative training file:");
		speciesNegTrainBuildField = new JTextField(18);
		speciesNegTrainBuildButton = new JButton("Search file");
		JPanel snbf41Panel = new JPanel(/*new BorderLayout()*/); snbf41Panel.add(speciesNameBuildNegLabel/*,BorderLayout.CENTER*/);
		JPanel snbf42Panel = new JPanel(); snbf42Panel.add(speciesNegTrainBuildField);
		JPanel snbf43Panel = new JPanel(); snbf43Panel.add(speciesNegTrainBuildButton);
		snbf4Panel.add(snbf41Panel);
		snbf4Panel.add(snbf42Panel);
		snbf4Panel.add(snbf43Panel);
		snbf4Panel.setBorder(BorderFactory.createEtchedBorder());
		
		JLabel locationTrainBuildLabel = new JLabel("<html>Location of function site " +
				                                      "in training data:</html>");
		locationTrainBuildField = new JTextField(18);
		JLabel sdfLabel 	= new JLabel(" ");
		JPanel snbf51Panel = new JPanel(new BorderLayout()); snbf51Panel.add(locationTrainBuildLabel,BorderLayout.CENTER);
		JPanel snbf52Panel = new JPanel(new GridLayout(3,1)); snbf52Panel.add(new JLabel(" "));snbf52Panel.add(locationTrainBuildField);snbf52Panel.add(new JLabel(" "));
		JPanel snbf53Panel = new JPanel(); snbf53Panel.add(sdfLabel);
		snbf5Panel.add(snbf51Panel);
		snbf5Panel.add(snbf52Panel);
		snbf5Panel.add(snbf53Panel);
		snbf5Panel.setBorder(BorderFactory.createEtchedBorder());
		
		speciesNameBuildField.getDocument().addDocumentListener(this);
		speciesPosTrainBuildField.getDocument().addDocumentListener(this);
		speciesNegTrainBuildField.getDocument().addDocumentListener(this);
		locationTrainBuildField.getDocument().addDocumentListener(this);
		speciesPosTrainBuildButton.addActionListener(this);
		speciesNegTrainBuildButton.addActionListener(this);
		
		speciesBuildPanel.add(snbf1Panel);
		speciesBuildPanel.add(snbf2Panel);
		speciesBuildPanel.add(snbf3Panel);
		speciesBuildPanel.add(snbf4Panel);
		speciesBuildPanel.add(snbf5Panel);
		
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//actions build panel
		actionsBuildPanel 				= new JPanel(new GridLayout(3,1));
		JLabel actionsBuildLabel		= 
			new JLabel("<html>Please provide the necessary information concerning the type of " +
					   "classification that needs to be performed. If the \"Perform optimization\" " +
					   "-option is selected, the building time of the classification model will " +
					   "increase 100-fold or more (depending on the parameters). See the manual for " +
					   "further information on optimizations.</html>");
		JPanel actionsBuildPanel1		= new JPanel(new BorderLayout());
		actionsBuildPanel1.add(actionsBuildLabel,BorderLayout.CENTER);
		
		JPanel actionsBuildPanel2		= new JPanel(new GridLayout(3,1));
		JPanel actionsBuildPanel21	 	= new JPanel(new GridLayout(1,3));
		JPanel actionsBuildPanel22		= new JPanel(new GridLayout(1,3));
		JPanel actionsBuildPanel23	 	= new JPanel();
		JLabel actionsBuildNameLabel	= new JLabel("<html>Name of the new action" +
													" (e.g. Donor GT classification)</html>");
		actionsBuildName				= new JTextField(20);
		JPanel actionsBuildPanel211		= new JPanel(new BorderLayout());
		JPanel actionsBuildPanel212		= new JPanel();
		actionsBuildPanel211.add(actionsBuildNameLabel,BorderLayout.CENTER);
		actionsBuildPanel212.add(actionsBuildName);
		actionsBuildPanel21.add(actionsBuildPanel211);
		actionsBuildPanel21.add(actionsBuildPanel212);
		JLabel actionsBuildClassifNameLabel = new JLabel("<html>Name of the classification " +
														" (e.g. donor)</html>");
		actionsBuildClassifName	 		= new JTextField(20);
		JPanel actionsBuildPanel221		= new JPanel(new BorderLayout());
		JPanel actionsBuildPanel222		= new JPanel();
		actionsBuildPanel221.add(actionsBuildClassifNameLabel,BorderLayout.CENTER);
		actionsBuildPanel222.add(actionsBuildClassifName);
		actionsBuildPanel22.add(actionsBuildPanel221);
		actionsBuildPanel22.add(actionsBuildPanel222);
		actionsBuildName.getDocument().addDocumentListener(this);
		actionsBuildClassifName.getDocument().addDocumentListener(this);
		
		actionsBuildOptimization		= new JCheckBox("Perform optimization (min)(max)(step)",false);
		actionsBuildOptimization.addActionListener(this);
		actionsBuildOptField1			= new JTextField(10); 
		actionsBuildOptField2			= new JTextField(10);
		actionsBuildOptField3			= new JTextField(10);
		actionsBuildOptField1.setEnabled(false);
		actionsBuildOptField2.setEnabled(false);
		actionsBuildOptField3.setEnabled(false);
		actionsBuildPanel23.add(actionsBuildOptimization);
		actionsBuildPanel23.add(actionsBuildOptField1);
		actionsBuildPanel23.add(actionsBuildOptField2);
		actionsBuildPanel23.add(actionsBuildOptField3);		
		actionsBuildPanel2.add(actionsBuildPanel21);
		actionsBuildPanel2.add(actionsBuildPanel22);
		actionsBuildPanel2.add(actionsBuildPanel23);

		JPanel actionsBuildPanel3		= new JPanel(new GridLayout(1,2));
		JPanel actionsBuildPanel31		= new JPanel(new BorderLayout());
		JPanel actionsBuildPanel32		= new JPanel(new GridLayout(0,1));
		JLabel actionsBuildEditLabel	= new JLabel("<html>By editing the classification features" +
				" you change the way of how the classification" +
				" model is build. Refer to manual for more info.</html>");
		editClassificationActions		= new JButton("Edit classification features");
		editClassificationActions.addActionListener(this);
		actionsBuildPanel31.add(actionsBuildEditLabel,BorderLayout.CENTER);
		actionsBuildPanel32.add(new JLabel(" "));
		actionsBuildPanel32.add(editClassificationActions);
		actionsBuildPanel32.add(new JLabel(" "));
		actionsBuildPanel3.add(actionsBuildPanel31);
		actionsBuildPanel3.add(actionsBuildPanel32);
		
		actionsBuildPanel2.setBorder(BorderFactory.createEtchedBorder());
		actionsBuildPanel3.setBorder(BorderFactory.createEtchedBorder());
		
		actionsBuildPanel.add(actionsBuildPanel1);
		actionsBuildPanel.add(actionsBuildPanel2);
		actionsBuildPanel.add(actionsBuildPanel3);
		
		//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		//finish build panel
		finishBuildPanel 	= new JPanel(new GridLayout(6,1));
		JPanel finishBuildPanel1 = new JPanel(new GridLayout(1,3));
		JPanel finishBuildPanel2 = new JPanel(new GridLayout(1,3));
		JPanel finishBuildPanel3 = new JPanel(new GridLayout(1,3));
		JPanel finishBuildPanel4 = new JPanel(new GridLayout(1,3));
		JPanel finishBuildPanel5 = new JPanel(new GridLayout(1,3));
		JPanel finishBuildPanel6 = new JPanel(new GridLayout(1,3));
		
		JPanel finishBuildPanel11 = new JPanel(new BorderLayout()); 
		applyFeatureSelection = new JCheckBox("Apply Feature Selection. This may increase accuracy.",false);
		finishBuildPanel11.add(applyFeatureSelection,BorderLayout.CENTER);
		finishBuildPanel1.add(finishBuildPanel11);
		
		JPanel finishBuildPanel21 = new JPanel(new BorderLayout());
		createEvaluateFile = new JCheckBox("Create corresponding evaluation file.",false);
		finishBuildPanel21.add(createEvaluateFile,BorderLayout.CENTER);
		finishBuildPanel2.add(finishBuildPanel21);
		
		JPanel finishBuildPanel31 = new JPanel(new BorderLayout());
		JPanel finishBuildPanel32 = new JPanel(new GridLayout(0,1));
		posAmountTrainingCheck = new JCheckBox("<html>Amount of positive training data to be used. " +
				" If not selected, all positive data is used.</html>",false);
		posAmountTrainingField = new JTextField(10);
		posAmountTrainingField.getDocument().addDocumentListener(this);
		finishBuildPanel31.add(posAmountTrainingCheck,BorderLayout.CENTER);
		finishBuildPanel32.add(new JLabel(" "));
		finishBuildPanel32.add(posAmountTrainingField);
		finishBuildPanel32.add(new JLabel(" "));
		finishBuildPanel3.add(finishBuildPanel31);
		finishBuildPanel3.add(finishBuildPanel32);
		
		JPanel finishBuildPanel41 = new JPanel(new BorderLayout());
		JPanel finishBuildPanel42 = new JPanel(new GridLayout(0,1));
		negAmountTrainingCheck = new JCheckBox("<html>Amount of negative training data to be used. " +
				" If not selected, all negative data is used.</html>",false);
		negAmountTrainingField = new JTextField(10);
		negAmountTrainingField.getDocument().addDocumentListener(this);
		finishBuildPanel41.add(negAmountTrainingCheck,BorderLayout.CENTER);
		finishBuildPanel42.add(new JLabel(" "));
		finishBuildPanel42.add(negAmountTrainingField);
		finishBuildPanel42.add(new JLabel(" "));
		finishBuildPanel4.add(finishBuildPanel41);
		finishBuildPanel4.add(finishBuildPanel42);
		
		JPanel finishBuildPanel51 = new JPanel(new BorderLayout());
		JPanel finishBuildPanel52 = new JPanel(new GridLayout(0,1));
		JLabel buildConfigNameLabel = new JLabel("<html>Please provide a name for the configuration file</html>");
		finishBuildConfigNameField = new JTextField(30);
		finishBuildPanel51.add(buildConfigNameLabel,BorderLayout.CENTER);
		finishBuildPanel52.add(new JLabel(" "));
		finishBuildPanel52.add(finishBuildConfigNameField);
		finishBuildPanel52.add(new JLabel(" "));
		finishBuildPanel5.add(finishBuildPanel51);
		finishBuildPanel5.add(finishBuildPanel52);
		
		JPanel finishBuildPanel61 = new JPanel(new BorderLayout()); 
		addToModelFileCheck = new JCheckBox("Add resulting configuration file and model to list of available classification models?",false);
		finishBuildPanel61.add(addToModelFileCheck,BorderLayout.CENTER);
		finishBuildPanel6.add(finishBuildPanel61);
		
		addToModelFileCheck.setEnabled(false);
		createEvaluateFile.setEnabled(false);
		
		finishBuildPanel1.setBorder(BorderFactory.createEtchedBorder());
		finishBuildPanel2.setBorder(BorderFactory.createEtchedBorder());
		finishBuildPanel3.setBorder(BorderFactory.createEtchedBorder());
		finishBuildPanel4.setBorder(BorderFactory.createEtchedBorder());
		finishBuildPanel5.setBorder(BorderFactory.createEtchedBorder());
		finishBuildPanel6.setBorder(BorderFactory.createEtchedBorder());
		
		finishBuildPanel.add(finishBuildPanel1);
		finishBuildPanel.add(finishBuildPanel2);
		finishBuildPanel.add(finishBuildPanel3);
		finishBuildPanel.add(finishBuildPanel4);
		finishBuildPanel.add(finishBuildPanel5);
		finishBuildPanel.add(finishBuildPanel6);
		
	}

	public void setMainPanel(){
		mainPanel.removeAll();
		switch(state){
		case BUILD_OR_USE 	:
			mainPanel.add(buildOrUsePanel,BorderLayout.CENTER);
			break;
		case SPECIES_USE		:
			mainPanel.add(speciesUsePanel,BorderLayout.CENTER);
			break;
		case ACTIONS_USE		:
			actionsUse = getPossibleActions(selectedUseSpecies);
			actionsUseList.setListData(actionsUse);
			actionsUseList.repaint();
			mainPanel.add(actionsUsePanel,BorderLayout.CENTER);
			break;
		case FINAL_USE			:
			if(selectedUseAction.length==1)
				finalUseMergeCheck.setEnabled(false);
			else
				finalUseMergeCheck.setEnabled(true);
			mainPanel.add(finalUsePanel,BorderLayout.CENTER);
			break;
		case SPECIES_BUILD		:
			mainPanel.add(speciesBuildPanel,BorderLayout.CENTER);
			break;
		case ACTIONS_BUILD		:
			mainPanel.add(actionsBuildPanel,BorderLayout.CENTER);
			break;
		case FINAL_BUILD		: 
			mainPanel.add(finishBuildPanel,BorderLayout.CENTER);
			break;
		default				: break;
		}
		
		mainPanel.add(buttonPanel,BorderLayout.SOUTH);
		this.setContentPane(mainPanel);
	}
	
	public void setButtons(){
		previousButton.setEnabled(true);
		nextButton.setEnabled(true);
		finishButton.setEnabled(true);
		cancelButton.setEnabled(true);
		switch(state){
		case BUILD_OR_USE :
			previousButton.setEnabled(false);
			finishButton.setEnabled(false);
			break;
		case SPECIES_USE :
			finishButton.setEnabled(false);
			if(speciesUseList.getSelectedIndex()==-1)
				nextButton.setEnabled(false);
			break;
		case ACTIONS_USE :
			finishButton.setEnabled(false);
			if(actionsUseList.getSelectedIndex()==-1)
				nextButton.setEnabled(false);
			break;
		case FINAL_USE :
			nextButton.setEnabled(false);
			break;
		case SPECIES_BUILD:
			finishButton.setEnabled(false);
			if(!speciesBuildComplete)
				nextButton.setEnabled(false);
			break;
		case ACTIONS_BUILD :
			finishButton.setEnabled(false);
			if(!actionsBuildComplete)
				nextButton.setEnabled(false);
			break;
		case FINAL_BUILD:
			nextButton.setEnabled(false);
			break;
		default : break;
		}
	}
	
	public void createFinalUseConfig(){
		//part 0: create the fileName for the configfile
		String configFileName = null;
		if(FunSiPJar.RUN_FROM_JAR){
			configFileName = FunSiPJar.USER_DIR+"config/";
			new File(configFileName).mkdir();
			if(finalUseConfigFileNameField.getText().equals("")){
				String message = "You have to provide a name for the configuration file.";
				JOptionPane.showMessageDialog(this, message, "Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			else{
				configFileName+=finalUseConfigFileNameField.getText();
			}
		}
		else{
			configFileName = "./config/";
			new File(configFileName).mkdir();
			if(finalUseConfigFileNameField.getText().equals("")){
				String message = "You have to provide a name for the configuration file.";
				JOptionPane.showMessageDialog(this, message, "Error",JOptionPane.ERROR_MESSAGE);
				return;
			}
			else{
				configFileName+=finalUseConfigFileNameField.getText();
			}
		}
		
		//part 1 : create the general options part of the configuration file
		Logger tempLogger = Logger.getLogger("tLog");
		tempLogger.addAppender(new NullAppender());
		FunSiPWorkflow fspw = new FunSiPWorkflow(tempLogger,configFileName);
		ParameterFileParser parser = new ParameterFileParser(tempLogger);
		
		//part 2 : locate the configuration file
		for(int i=0;i<selectedUseAction.length;i++){
			String act = selectedUseAction[i];
			String configFile = getModelLocation(selectedUseSpecies,act);
			FunSiPWorkflow fw = parser.parseFile(configFile);
			List<ClassificationAction> loc_actions = fw.getClassificationActions();
			for(int j=0;j<loc_actions.size();j++){
				fspw.addClassificationAction(loc_actions.get(j));
			}
		}
		
		//part 3 : add the files to all classification actions
		if(finalUseFiles!=null && finalUseFiles.length!=0){
			File[] files = new File[finalUseFiles.length];
			for(int i=0;i<finalUseFiles.length;i++)
				files[i] = new File(finalUseFiles[i]);
			fspw.setExplicitFiles(files);
		}
		else{
			String message = 
				"Warning. There are no fasta-files provided that are to be \n" +
				"evaluated by FunSiP. If you continue, the resulting configurationfile \n" +
				"will be correctly formed but also be disfunctional unless these \n" +
				"fasta-files are provided at the commandline. Continue (Y/N)?";
			int res = JOptionPane.showConfirmDialog(this, message, "Warning", JOptionPane.YES_NO_OPTION);
			if(res!=JOptionPane.YES_OPTION){
				return;	//return to finish-use panel
			}
		}
		
		//part 4 : write the resulting workload-object to the configuration file.
		try{
			String t = ParameterFileParser.getConfigurationFileString(fspw);
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(configFileName)));
			writer.write(t);
			writer.close();
		}
		catch(Exception exc){
			JOptionPane.showMessageDialog(this, "Error while creating configuration file : "+exc, "Error",JOptionPane.ERROR_MESSAGE);
			this.dispose();
		}
		
		String message = "The configuration file has been succesfully created.\n " +
				"To run FunSiP with this configurationfile, please select the \n" +
				"'launch'-option. The results of the evaluation(s) will be written \n" +
				"to the indicated directory.";
		JOptionPane.showMessageDialog(this, message,"Wizard complete",JOptionPane.INFORMATION_MESSAGE);
		fspParent.setNewConfiguration(configFileName);
		this.dispose(); //Very weird. Calling this method puts the CPU on a continuous 100%
		//this.setVisible(false);
	}
	
	public String getResultingConfigurationFileName(){
		return finalConfigurationFileName;
	}
	
	
	public void createFinalBuildConfig(){
		if(trainLocation<0){
			JOptionPane.showMessageDialog(this,"Location of indicated functional site is lower than 0","Illegal parameter",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if(posAmountTrainingCheck.isEnabled()&&posAmountTraining<0){
			JOptionPane.showMessageDialog(this,"Amount of positive training data is lower than 0","Illegal parameter",JOptionPane.ERROR_MESSAGE);
			return;
		}
		if(negAmountTrainingCheck.isEnabled()&&negAmountTraining<0){
			JOptionPane.showMessageDialog(this,"Amount of negative training data is lower than 0","Illegal parameter",JOptionPane.ERROR_MESSAGE);
			return;
		}
		String configName = finishBuildConfigNameField.getText().trim();
		if(configName.equals("")){
			JOptionPane.showMessageDialog(this,"You must provide a name for the configuration file.","Blank parameter",JOptionPane.ERROR_MESSAGE);
			return;
		}
		
		//create the workflow object which will perform the necessary actions.
		//then write the object to a configuration file.
		Logger tempLogger = Logger.getLogger("TempLogger");
		tempLogger.addAppender(new NullAppender());
		FunSiPWorkflow flow = new FunSiPWorkflow(tempLogger,configName);
		ClassificationAction action = new ClassificationAction(tempLogger,flow);
		
		action.setName(actionsBuildName.getText());
		action.setClassification_name(actionsBuildClassifName.getText());
		action.setClassifier_type(ClassificationAction.CLASSIFIERS.WEKASVM);
		action.setPositive_training_file(speciesPosTrainBuildField.getText());
		action.setNegative_training_file(speciesNegTrainBuildField.getText());
		action.setTraining_functional_site_position(trainLocation);
		action.setCommand(ClassificationAction.COMMAND.BUILD_MODEL);
		action.setModel_file(action.getClassification_name()+".model");
		action.setFeatures_output_file(action.getClassification_name()+".arff");
		action.setClassificationFeatures(this.buildClassificationAction.getClassificationFeatures());
		if(applyFeatureSelection.isSelected()){
			action.getFeatureSelection().setFeature_selection_purpose(FeatureSelectionStub.FEATURE_SELECTION_PURPOSE.USE_AND_WRITE_FULL_FILTER);
			action.getFeatureSelection().setFeature_selection_type(FeatureSelectionStub.FEATURE_SELECTION_TYPE.SYMMETRICAL_UNCERTAINTY);
			//action.setFeatures_output_file(action.getClassification_name()+"_filter.txt");
			action.getFeatureSelection().setFeature_selection_output_file(action.getClassification_name()+"_filter.txt");
		}
		action.setPositive_training_amount(posAmountTraining);
		action.setNegative_training_amount(negAmountTraining);
		action.setOutputdirectory("./output/");
		
		if(actionsBuildOptimization.isSelected()){
			
			try{
				int min = Integer.parseInt(actionsBuildOptField1.getText().trim());
				int max = Integer.parseInt(actionsBuildOptField2.getText().trim());
				int step = Integer.parseInt(actionsBuildOptField3.getText().trim());
				flow.setOptimizer(true);
				flow.setOptimizer_steps(min, max, step);
				
				action.setCrossvalidation_fold(10);			//not really necessary, but will be used anyway
				action.setComplexityCrossvalidation(true);	//not really necessary, but will be used anyway
			}
			catch(Exception exc){
				JOptionPane.showMessageDialog(this,"Unable to set the correct optimization parameters.\n " +
						"Please check if the supplied values are integers only.", "Cannot set optimization", JOptionPane.ERROR_MESSAGE);
				return;
			}
		}
		
		flow.addClassificationAction(action);
		String t = ParameterFileParser.getConfigurationFileString(flow);
		String configFileName = null;
		try{			
			if(FunSiPJar.RUN_FROM_JAR){
				configFileName = FunSiPJar.USER_DIR+"config/";
				new File(configFileName).mkdir();
			}
			else{
				configFileName = "./config/";
				new File(configFileName).mkdir();
			}
			configFileName+= finishBuildConfigNameField.getText();
			BufferedWriter writer  = new BufferedWriter(new FileWriter(new File(configFileName)));
			writer.write(t);
			writer.close();
		}
		catch(Exception exc){
			JOptionPane.showMessageDialog(this,"Unable to write the configuration file to disk. ", "Cannot create configuration file", JOptionPane.ERROR_MESSAGE);
			this.dispose();
		}
		
		String message = "The configuration file has been succesfully created.\n " +
		"To run FunSiP with this configurationfile, please select the \n" +
		"'launch'-option. The results of the evaluation(s) will be written \n" +
		"to the indicated directory.";
		JOptionPane.showMessageDialog(this, message,"Wizard complete",JOptionPane.INFORMATION_MESSAGE);
		fspParent.setNewConfiguration(configFileName);
		this.dispose();
	}
	
	public void actionPerformed(ActionEvent e){
		//System.out.println("previous state : "+state);
		if(e.getSource().equals(nextButton)){
			if(state==WIZARD_STATE.BUILD_OR_USE){
				if(buildRadioButton.isSelected())
					state = WIZARD_STATE.SPECIES_BUILD;
				else
					state = WIZARD_STATE.SPECIES_USE;
			}
			else{
				state = WIZARD_STATE.next(state);
			}
		}
		else if(e.getSource().equals(previousButton)){
			state = WIZARD_STATE.previous(state);
		}
		else if(e.getSource().equals(cancelButton)){
			String message = "Exit FunSiP wizard (Y/N)?";
			int sel = JOptionPane.showConfirmDialog(this,message,"Exit wizard?",JOptionPane.YES_NO_OPTION);
			if(sel==JOptionPane.YES_OPTION){
				this.dispose();
			}
		}
		else if(e.getSource().equals(finishButton)){
			if(state==WIZARD_STATE.FINAL_USE){
				createFinalUseConfig();
			}
			else if(state==WIZARD_STATE.FINAL_BUILD){
				createFinalBuildConfig();
			}
		}
		
		//add sequence file to be examined
		else if(e.getSource().equals(finalUseAddFileButton)){
			JFileChooser chooser = new JFileChooser(".");
			chooser.setMultiSelectionEnabled(true);
			chooser.showOpenDialog(this);
			File[] files = chooser.getSelectedFiles();
			String[] filePaths = new String[files.length];
			for(int i=0;i<files.length;i++)
				filePaths[i] = files[i].getAbsolutePath();
			finalUseFiles = filePaths;
			finalUseFileList.setListData(filePaths);
		}
		//remove sequence file to be examined
		else if(e.getSource().equals(finalUseRemoveFileButton)){
			if(finalUseFileList.getSelectedIndex()!=-1){
				List<String> newData = new ArrayList<String>();
				for(int i=0;i<finalUseFiles.length;i++){
					if(i!=finalUseFileList.getSelectedIndex()){
						newData.add(finalUseFiles[i]);
					}
				}
				String[] newDat = new String[newData.size()];
				for(int i=0;i<newData.size();i++){
					newDat[i]=newData.get(i);
				}
				this.finalUseFiles = newDat;
				finalUseFileList.setListData(finalUseFiles);
			}
		}
		
		//find the positive training file
		else if(e.getSource().equals(speciesPosTrainBuildButton)){
			JFileChooser chooser = new JFileChooser();
			chooser.setMultiSelectionEnabled(false);
			int t = chooser.showOpenDialog(this);
			if(t == JFileChooser.APPROVE_OPTION){
				File file = chooser.getSelectedFile();
				String filePath = file.getAbsolutePath();
				speciesPosTrainBuildField.setText(filePath);
			}
		}
		//find the negative training file
		else if(e.getSource().equals(speciesNegTrainBuildButton)){
			JFileChooser chooser = new JFileChooser();
			chooser.setMultiSelectionEnabled(false);
			int t = chooser.showOpenDialog(this);
			if(t == JFileChooser.APPROVE_OPTION){
				File file = chooser.getSelectedFile();
				String filePath = file.getAbsolutePath();
				speciesNegTrainBuildField.setText(filePath);
			}
		}
		else if(e.getSource().equals(editClassificationActions)){
			JPanel pane = new SprClassificationFeaturePanel(buildClassificationAction,new Dimension(500,200));
			JOptionPane.showMessageDialog(this,pane,"Please provide classification features",JOptionPane.INFORMATION_MESSAGE);
		}
		else if(e.getSource().equals(actionsBuildOptimization)){
			if(actionsBuildOptimization.isSelected()){
				actionsBuildOptField1.setEnabled(true);
				actionsBuildOptField2.setEnabled(true);
				actionsBuildOptField3.setEnabled(true);
			}
			else{
				actionsBuildOptField1.setEnabled(false);
				actionsBuildOptField2.setEnabled(false);
				actionsBuildOptField3.setEnabled(false);
			}
		}
		
		//System.out.println("new state : "+state);
		setMainPanel();
		setButtons();
	}
	
	private void testTextFields(DocumentEvent e){
		if(speciesNameBuildField.getText().equals("")||
				speciesPosTrainBuildField.getText().equals("")||
				speciesNegTrainBuildField.getText().equals("")||
				locationTrainBuildField.getText().equals("")){
			
			speciesBuildComplete = false;
		}
		else{
			speciesBuildComplete = true;
		}
		
		if(actionsBuildName.getText().equals("")||
				actionsBuildClassifName.getText().equals("")){
			actionsBuildComplete = false;
		}
		else{
			actionsBuildComplete = true;
		}
			
		
		setButtons();
	}
	
	public void changedUpdate(DocumentEvent e){testTextFields(e);}
	public void insertUpdate(DocumentEvent e){
		if(e.getDocument().equals(locationTrainBuildField.getDocument())){
			String t = locationTrainBuildField.getText();
			try{
				try{
					trainLocation = Integer.parseInt(t);
				}
				catch(Exception exc){
					JOptionPane.showMessageDialog(this,"The location must be an integer value", "Cannot parse number", JOptionPane.ERROR_MESSAGE);
					EventQueue.invokeLater(new Runnable()
					{public void run() {
						locationTrainBuildField.setText("");
					}
					});
				}
			}
			catch(Exception exc){
				//can happen that the user types to fast for the eventhandler to throw the
				//showmessage, causing an illegalstateException.
			}
		}
		else if(e.getDocument().equals(posAmountTrainingField.getDocument())){
			String t = posAmountTrainingField.getText();
			try{
				try{
					posAmountTraining = Integer.parseInt(t);
				}
				catch(Exception exc){
					JOptionPane.showMessageDialog(this,"The parameter must be an integer value", "Cannot parse number", JOptionPane.ERROR_MESSAGE);
					EventQueue.invokeLater(new Runnable()
					{public void run() {
						posAmountTrainingField.setText("");
					}
					});
				}
			}
			catch(Exception exc){
				//can happen that the user types to fast for the eventhandler to throw the
				//showmessage, causing an illegalstateException.
			}
		}
		else if(e.getDocument().equals(negAmountTrainingField.getDocument())){
			String t = negAmountTrainingField.getText();
			try{
				try{
					negAmountTraining = Integer.parseInt(t);
				}
				catch(Exception exc){
					JOptionPane.showMessageDialog(this,"The parameter must be an integer value", "Cannot parse number", JOptionPane.ERROR_MESSAGE);
					EventQueue.invokeLater(new Runnable()
					{public void run() {
						negAmountTrainingField.setText("");
					}
					});
				}
			}
			catch(Exception exc){
				//can happen that the user types to fast for the eventhandler to throw the
				//showmessage, causing an illegalstateException.
			}
		}
		testTextFields(e);
	}
	public void removeUpdate(DocumentEvent e){testTextFields(e);}
	
	public void valueChanged(ListSelectionEvent e){
		if(e.getSource().equals(speciesUseList)&&speciesUseList.getSelectedIndex()!=-1){
			selectedUseSpecies = (String) speciesUseList.getSelectedValue();
		}
		if(e.getSource().equals(actionsUseList)&&actionsUseList.getSelectedIndex()!=-1){
			Object [] temp = actionsUseList.getSelectedValues();
			String[] t = new String[temp.length];
			for(int i=0;i<temp.length;i++)
				t[i] = (String)temp[i];
			selectedUseAction = t;
		}
		setButtons();
	}
	
	public void setDimensions(Dimension d, Component f){
		f.setPreferredSize(d);
		f.setMaximumSize(d);
		f.setMinimumSize(d);
	}
	
	
	public void windowClosed(WindowEvent e){
		//this.dispose();
		this.setVisible(false);
	}
	public void windowClosing(WindowEvent e){
		windowClosed(e);
	}
	public void windowActivated(WindowEvent e){
	}
	public void windowDeactivated(WindowEvent e){
	}
	public void windowIconified(WindowEvent e){
		windowDeiconified(e);
	}
	public void windowDeiconified(WindowEvent e){
	}
	public void windowOpened(WindowEvent e){	
	}
	public void windowGainedFocus(WindowEvent e){
	}
	public void windowLostFocus(WindowEvent e){
		this.requestFocus();
	}
	
	
}


