package main;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import main.FunSiPFeatureExplorer.Combination;

public class FunSiPFeatureGui implements ActionListener {

	private JList list = null;
	private DrawPanel drawpanel = null;
	private JButton button = null;
	
	public static void main(String[] args){
		if(args.length==0){
			System.out.println("No file with results detected.");
			return;
		}
		else{
			List<Combination> combinations = readCombinationsFromFile(args[0]);
			new FunSiPFeatureGui(combinations);
		}
	}
	private static List<Combination> readCombinationsFromFile(String s){
		return null;
	}
	
	public FunSiPFeatureGui(List<Combination> combinations){
		JFrame frame = new JFrame("Display of feature results");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		JFrame.setDefaultLookAndFeelDecorated(true);
		
		JPanel content = createContent(combinations);
		frame.setContentPane(content);
		
		frame.setVisible(true);
		frame.pack();
		frame.setLocationRelativeTo(null);
	}
	
	public void setDimensions(Container c, Dimension d){
		c.setMinimumSize(d);
		c.setMaximumSize(d);
		c.setPreferredSize(d);
	}
	
	public JPanel createContent(List<Combination> combinations){
		JPanel panel = new JPanel();
		setDimensions(panel,new Dimension(800,600));
		
		JPanel leftPanel = createLeftPanel(combinations);
		JPanel rightPanel = createRightPanel();
		
		panel.add(leftPanel);
		panel.add(rightPanel);
		
		return panel;
	}
	
	
	public JPanel createRightPanel(){
		drawpanel = new DrawPanel();
		setDimensions(drawpanel,new Dimension(590,600));
		drawpanel.setBorder(BorderFactory.createEtchedBorder());
		return drawpanel;
	}
	
	public JPanel createLeftPanel(List<Combination> combinations){
		Combination[] combs = getArray(combinations);
		list =  new JList(combs);
		button = new JButton("show");
		JPanel leftPanel = new JPanel(new BorderLayout());
		setDimensions(leftPanel,new Dimension(200,600));
		leftPanel.add(new JScrollPane(list),BorderLayout.CENTER);
		leftPanel.add(button,BorderLayout.SOUTH);
		button.addActionListener(this);
		return leftPanel;
	}
	
	private int getLocation(int pos, String ud){
		if(ud.equals("up"))
			return 20-pos;
		else if(ud.equals("down"))
			return 20+pos;
		else
			return pos;
			
	}
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(button)){
		//	System.out.println("Button pressed");
			if(list.getSelectedIndex()!=-1){
			//	System.out.println("list index is not null");
				Combination comb = (Combination) list.getSelectedValue();
				String[] des1 = comb.description_1.split(" ");
				String[] des2 = comb.description_2.split(" ");
				List<Integer> firsts = new ArrayList<Integer>();
				List<Integer> seconds = new ArrayList<Integer>();
				
				if(des1[1].length()==1){
					firsts.add(getLocation(Integer.parseInt(des1[2]),des1[3]));
				}
				else if(des1[1].length()==2){
					firsts.add(getLocation(Integer.parseInt(des1[2]),des1[3]));
					firsts.add(getLocation(Integer.parseInt(des1[2])+1,des1[3]));
				}
				
				if(des2[1].length()==1){
					seconds.add(getLocation(Integer.parseInt(des2[2]),des2[3]));
				}
				else if(des2[1].length()==2){
					seconds.add(getLocation(Integer.parseInt(des2[2]),des2[3]));
					seconds.add(getLocation(Integer.parseInt(des2[2])+1,des2[3]));
				}
				
				drawpanel.setLocations(firsts, seconds);
				drawpanel.setDescriptions(comb.description_1, comb.description_2, comb.support);
			}
			drawpanel.repaint();
			//draw the locations on the screen
		}
	}
	
	private Combination[] getArray(List<Combination> combinations){
		Combination[] result = new Combination[combinations.size()];
		for(int i=0;i<combinations.size();i++)
			result[i] = combinations.get(i);
		return result;
	}
	
	private class DrawPanel extends JPanel{
		
		public static final long serialVersionUID = 132456l;
		
		private List<Integer> firstLocs = new ArrayList<Integer>();
		private List<Integer> secondLocs = new ArrayList<Integer>();
		
		private String description_1 = "";
		private String description_2 = "";
		
		private double support = 0.0;
		
		public DrawPanel(){
			super();
		}
		
		public void paintComponent(Graphics g){
			super.paintComponent(g);
			
			Graphics2D g2d = (Graphics2D)g;
			//g2d.drawRect(50, 300, 450, 20);
			
			//first fill, then draw rectangles around them.
			g2d.setColor(Color.BLUE);
			for(int i=0;i<firstLocs.size();i++){
				g2d.fillRect(50+firstLocs.get(i)*10, 300, 10, 10);
				//System.out.println(firstLocs.get(i));
			}
			g2d.setColor(Color.GREEN);
			for(int i=0;i<secondLocs.size();i++){
				g2d.fillRect(50+secondLocs.get(i)*10, 300, 10, 10);
			}
			
			if(firstLocs.size()>0 && secondLocs.size()>0){
				int xpos1 = 50+firstLocs.get(0)*10 +5;
				int xpos2 = 50+secondLocs.get(0)*10 +5;
				
				
				g2d.setColor(Color.BLACK);
				g2d.drawString(description_1, xpos1, 150);
				g2d.drawString(description_2, xpos2, 450);
				
				g2d.setColor(Color.BLUE);
				g2d.drawLine(xpos1, 150, xpos1, 300);
				g2d.setColor(Color.GREEN);
				g2d.drawLine(xpos2,450,xpos2,300);
			}
			
			g2d.setColor(Color.BLACK);
			g2d.drawString("Support : "+support, 20, 20);
			for(int i=50;i<=450;i+=10){
				g2d.drawRect(i, 300, 10, 10);
			}
			
			g2d.drawLine(250, 250, 250, 350);
			g2d.drawString("splicesite",230, 360);
			
			g2d.drawLine(150, 290, 150, 320);
			g2d.drawLine(350, 290, 350, 320);
			
			g2d.setFont(new Font("Serif",Font.PLAIN,8));
			for(int i=0;i<20;i++){
				g2d.drawString(""+i,250+10*i+5,320);
				g2d.drawString(""+i, 250-10*i-5, 320);
			}
			
			
			
			
		}
		
		public void setLocations(List<Integer>first,List<Integer> second){
			firstLocs.clear();
			secondLocs.clear();
			firstLocs.addAll(new TreeSet<Integer>(first));
			secondLocs.addAll(new TreeSet<Integer>(second));
		
		}
		
		public void setDescriptions(String des1,String des2,double val){
			this.description_1 = des1;
			this.description_2 = des2;
			this.support = val;
			
			if(this.description_1.startsWith("P")){
				description_1 = description_1.substring(2,description_1.length());
			}
			
			if(this.description_2.startsWith("P")){
				description_2 = description_2.substring(2,description_2.length());
			}
		}
	}
	
}
