package test;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.text.NumberFormat;
import java.util.List;
import java.util.Locale;

import util.io.FileParser;

public class FindSupport {

	
	public static void main(String[] args){
		if(args.length==1){
			try{
				//rnafold output convert to normale secondary structure output
				BufferedReader reader = new BufferedReader(new FileReader(new File(args[0])));
				BufferedWriter writer = new BufferedWriter(new FileWriter(new File(args[0]+"2")));
				String s = reader.readLine();
				while(s!=null){
					if(s.contains("N")){
						//do nothing, just read secondary structure
						s = reader.readLine();
					}
					else{
						s = reader.readLine(); //read true secondary structure
						writer.write(s+"\n");
					}
					s = reader.readLine();	//read again the sequence.
				}
				reader.close();
				writer.close();
			}
			catch(Exception exc){
				exc.printStackTrace();
				return;
			}
		}
		else if(args.length==2){
			if(!args[0].endsWith("/"))
				args[0] = args[0]+"/";
			new FindSupport(args[0],args[0]+"donors.pos.ss",args[0]+"donors.neg.ss",Integer.parseInt(args[1]));
			new FindSupport(args[0],args[0]+"acceptors.pos.ss",args[0]+"acceptors.neg.ss",Integer.parseInt(args[1]));
		}
		else if(args.length==3){
			String base = args[0].substring(args[0].lastIndexOf("/"),args[0].length());
			new FindSupport(base,args[0],args[1],Integer.parseInt(args[2]));
		}
		else{
			System.err.println("Incorrect parameters");
		}
	}
	
	public FindSupport(String baseDirectory,String posfile,String negfile,int splicesite){
		FileParser parser = new FileParser();
		try{
			String outFileName = posfile.substring(posfile.lastIndexOf("/"),posfile.lastIndexOf("."));
			if(!baseDirectory.endsWith("/"))
				baseDirectory = baseDirectory+"/";
			File outFile = new File(baseDirectory+outFileName+".txt");
			BufferedWriter writer = new BufferedWriter(new FileWriter(outFile));
			List<String> posdata = parser.extractSequencesFromFile(posfile);
			List<String> negdata = parser.extractSequencesFromFile(negfile);
			
			double[] posSupport = null;
			double[] negSupport = null;
			
			posSupport = findSupport(posdata,splicesite-10,20);
			negSupport = findSupport(negdata,splicesite-10,20);
			
			NumberFormat format  = NumberFormat.getInstance(Locale.FRENCH);
			format.setMaximumFractionDigits(2);
			
			for(int i=0;i<posSupport.length;i++)
				writer.write(format.format(posSupport[i])+"\t");
			writer.write("\n");
			for(int i=0;i<negSupport.length;i++)
				writer.write(format.format(negSupport[i])+"\t");
			writer.write("\n");
			writer.close();
			
			/*for(int i=0;i<posSupport.length;i++)
				System.out.print(format.format(posSupport[i])+"\t");
			System.out.println();
			for(int i=0;i<negSupport.length;i++)
				System.out.print(format.format(negSupport[i])+"\t");
			System.out.println("\n");
			*/
			
			
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.exit(1);
		}
	}
	
	
	public double[] findSupport (List<String> data,int start,int length){
		double[]result = new double[length];
		for(int i=0;i<length;i++){
			double temp = 0.0;
			for(int j=0;j<data.size();j++){
				String seq = data.get(j);
				char c = seq.charAt(start+i);
				if(c!='.')
					temp++;
			}
			temp = temp/(double)data.size();
			result[i] = temp;
		}
		return result;
	}
}
