package test.functions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import util.io.FileParser;

public class TestFunctions {
	
	
	
	public static void main(String[] args){
		if(args.length!=4){
			System.err.println("Needs 4 args : [positive-file] [negative-file] [site] [cutoff]");
			System.exit(1);
		}
		new TestFunctions(args);
	}
	
	public TestFunctions(String[] args){
		String positiveFileName = args[0];
		String negativeFileName = args[1];
		int site = Integer.parseInt(args[2]);
		double cutoff = Double.parseDouble(args[3]);
		
		try{
			FileParser parser = new FileParser();
			List<String> posData = parser.extractRandomSequencesFromFile(positiveFileName, -1);
			List<String> negData = parser.extractRandomSequencesFromFile(negativeFileName, -1);
			
			List<Double[]> posDoubleData = convertData(posData);
			List<Double[]> negDoubleData = convertData(negData);
			FunctionUtils fu = new FunctionUtils();
			int counter = 0;
			int num_vars = 7;
			int mul_vars = 3;
			while(true){
				Function f = fu.generateNextFunction(num_vars,mul_vars);
				double [] result = testFunction(posDoubleData,negDoubleData,f,site);
				double local_cutoff = result[0];
				if(local_cutoff>cutoff){
					System.out.println(f.toString()+" "+result[1]+" : "+local_cutoff+" :: positive : "+result[2]+" "+result[3]+" "+result[4]+" :: neg : "+result[5]+" "+result[6]+" "+result[7]);
					System.exit(0);
				}
				counter++;
				if(counter%10==0)
					System.out.println(counter+" functions tested with variables : "+num_vars+" and "+mul_vars);
				if(counter%100==0){
					num_vars++;
					mul_vars++;
				}
			}
			
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
	

	
	public double[] testFunction(List<Double[]> posData,List<Double[]>negData,Function f,int site){
		double[] result = new double[8];
		
		for(int i=40;i>5;i--){
			List<Double> posResults = new ArrayList<Double>();
			List<Double> negResults = new ArrayList<Double>();
			for(int j=0;j<posData.size();j++){
				Double[] data = posData.get(j);
				data = getCorrectData(data,i,site);
				double p = executeFunction(f,data);
				posResults.add(p);
			}
			for(int j=0;j<negData.size();j++){
				Double[] data = negData.get(j);
				data = getCorrectData(data,i,site);
				double p = executeFunction(f,data);
				negResults.add(p);
			}
			Collections.sort(posResults);
			Collections.sort(negResults);
			
			/*if(i==8){
				for(int j=0;j<posResults.size();j++){
					System.out.print(posResults.get(j)+" ");
				}
				System.out.println("\n--------------");
				for(int j=0;j<negResults.size();j++){
					System.out.print(negResults.get(j)+" ");
				}
				System.exit(1);
				
			}*/
			
			//now take median, max and min, then see how well the data of
			//negative and positive seperates. This will give a cutoff value,
			//which is to be put in the result

			double posMin = posResults.get(0);
			double negMin = negResults.get(0);
			double posMax = posResults.get(posResults.size()-1);
			double negMax = negResults.get(negResults.size()-1);
			double posMed = posResults.get(posResults.size()/2);
			double negMed = negResults.get(negResults.size()/2);
			
			if(posMin<negMin){
				if(posMed<negMin){
					if(posMax<negMin){
						result[0] = 1;
						result[1] = i;
						result[2] = posMin;
						result[3] = posMed;
						result[4] = posMax;
						result[5] = negMin;
						result[6] = negMed;
						result[7] = negMax;
					}
					else{
						double local_cutoff = 0;
						for(int j=posResults.size()/2;j<posResults.size();j++){
							if(posResults.get(j)>negMin){
								local_cutoff = (double)j/(double)posResults.size();
								break;
							}
						}
						if(local_cutoff > result[0]){
							result[0] = local_cutoff;
							result[1] = i;
							result[2] = posMin;
							result[3] = posMed;
							result[4] = posMax;
							result[5] = negMin;
							result[6] = negMed;
							result[7] = negMax;
						}
					}
				}
				else{
					//do nothing, values overlap too much.
				}
			}
			else{
				if(negMed<posMin){
					if(negMax<posMin){
						result[0] = 1;
						result[1] = i;
						result[2] = posMin;
						result[3] = posMed;
						result[4] = posMax;
						result[5] = negMin;
						result[6] = negMed;
						result[7] = negMax;
					}
					else{
						double local_cutoff = 0;
						for(int j=negResults.size()/2;j<negResults.size();j++){
							if(negResults.get(j)>posMin){
								local_cutoff = (double)j/(double)negResults.size();
								break;
							}
						}
						if(local_cutoff > result[0]){
							result[0] = local_cutoff;
							result[1] = i;
							result[2] = posMin;
							result[3] = posMed;
							result[4] = posMax;
							result[5] = negMin;
							result[6] = negMed;
							result[7] = negMax;
						}
					}
				}
				else{
					//do nothing, values overlap too much
				}
			}
		}
		return result;
	}
	
	public double executeFunction(Function f,Double[] data){
		double result = 0;
		for(int i=0;i<data.length-f.getNumberOfVariables();i++){
			double[] temp = new double[f.getNumberOfVariables()];
			for(int j=0;j<f.getNumberOfVariables();j++)
				temp[j] = data[i+j];
			double s = f.executeFunction(temp);
			double p = Math.sin(Math.PI*s);
			if(p>=0)
				result+=s;
			else
				result-=s;
		}
		return result;
	}
	
	
	public Double[] getCorrectData(Double[] data, int length,int site){
		List<Double> res = new ArrayList<Double>();
		for(int i=site-length;i<site+length;i++){
			res.add(data[i]);
		}
		Double [] result = new Double [res.size()];
		for(int i=0;i<res.size();i++)
			result[i] = res.get(i);
		return result;
	}
	
	private List<Double[]> convertData(List<String> data){
		List<Double[]> res = new ArrayList<Double[]>();
		
		for(int i=0;i<data.size();i++){
			String temp = data.get(i).toUpperCase().split(" ")[0].trim();
			Double [] x = new Double[temp.length()];
			for(int j=0;j<temp.length();j++){
				switch(temp.charAt(j)){
				case 'A' : x[j] = 135.14; break;
				case 'C' : x[j] = 111.11; break;
				case 'G' : x[j] = 151.14; break;
				case 'T' : case 'U' : x[j] = 112.092; break;
				default : x[j] = 127.3705; break;//gemiddelde
				}
			}
			res.add(x);
		}
		return res;
	}
	
	
}
