/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util;

public class FeatureExtractionExtension {

	
	public static final int U = 0; //-->volgorde : zie boek biochemie
	public static final int C = 1;
	public static final int A = 2;
	public static final int G = 3;
	
	public static final char[][][] AMINOACIDS = 
		{
		{{'F','F','L','L'},{'S','S','S','S'},{'Y','Y','O','O'},{'C','C','O','W'}},
		{{'L','L','L','L'},{'P','P','P','P'},{'H','H','Q','Q'},{'R','R','R','R'}},
		{{'I','I','I','M'},{'T','T','T','T'},{'N','N','K','K'},{'S','S','R','R'}},
		{{'V','V','V','V'},{'A','A','A','A'},{'D','D','E','E'},{'G','G','G','G'}}
		};
	
	
	public static void main(String[] args){
		String test = "AATTGCGGCTGACTCT";
		char[] temp = test.toCharArray();
		int[] t = convertToIntArray(temp);
		
		char[] aminos = dnaToAmino(t);
	
		String result = new String(aminos);
		System.out.println(result);
	}
	
	public static char[] dnaToAmino(String sequence){
		return dnaToAmino(sequence.toCharArray());
	}
	public static char[] dnaToAmino(char[] sequence){
		return dnaToAmino(convertToIntArray(sequence));
	}
	
	private static char[] dnaToAmino(int[] sequence){
		int length = sequence.length;
		char[] result = new char[length/3];
		for(int i=0;i<length && (i+3)<length;i+=3){
			int N1 = sequence[i];
			int N2 = sequence[i+1];
			int N3 = sequence[i+2];
			char res = AMINOACIDS[N1][N2][N3];
			result[i/3] = res;
		}
		return result;
	}
	
	public static int[] convertToIntArray(char[] array) throws IllegalArgumentException{
		int[] result = new int[array.length];
		int teller = 0;
		for(int i=0;i<array.length;i++){
			switch(array[i]){
			case 'A' : result[i]=A; break;
			case 'a' : result[i]=A;	break;
			case 'C' : result[i]=C; break;
			case 'c' : result[i]=C; break;
			case 'G' : result[i]=G; break;
			case 'g' : result[i]=G; break;
			case 'T' : result[i]=U; break;
			case 't' : result[i]=U; break;
			case 'U' : result[i]=U; break;
			case 'u' : result[i]=U; break;
			default : teller++;break;
			}
		}	
		if(teller>0)
			throw new IllegalArgumentException();
		return result;
	}
	
}
