/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

public class SMOSizeDataExtractor {

	public static void main(String[] args){
		int linesPos 		= 100;
		int linesNeg 		= 200;
		String directory1 	= "./data/ath/backup/";
		String directory2 	= "./data/ath/";
		boolean random 		= false;
		if(args.length!=0){
			try{
				directory1 = args[0];
				directory2 = args[1];
				linesPos = Integer.parseInt(args[2]);
				linesNeg = Integer.parseInt(args[3]);
				if(args.length==5 && args[4].toUpperCase().equals("RANDOM"))
					random = true;
			}
			catch(Exception exc){
				System.out.println(exc);
			}
		}
		
		new SMOSizeDataExtractor(directory1,directory2,linesPos,linesNeg,random);
	
	}
	
	public SMOSizeDataExtractor(String directoryFrom,String directoryTo, int linesPos,int linesNeg, boolean random){
		File fileNegAcc = new File(directoryFrom+"acceptors.neg");
		File filePosAcc = new File(directoryFrom+"acceptors.pos");
		File fileNegDon = new File(directoryFrom+"donors.neg");
		File filePosDon = new File(directoryFrom+"donors.pos");
		
		File destNegAcc = new File(directoryTo+"acceptors.neg");
		File destPosAcc = new File(directoryTo+"acceptors.pos");
		File destNegDon = new File(directoryTo+"donors.neg");
		File destPosDon = new File(directoryTo+"donors.pos");
	
		File dir = new File(directoryFrom);
		String[] fileNames = dir.list();
		boolean containsSS = false;
		for(String s : fileNames){
			if(s.endsWith(".ss")){
				containsSS = true;
				break;
			}
		}
			
		if(random){
			extractAndWriteRandom(fileNegAcc,destNegAcc,linesNeg,containsSS);
			extractAndWriteRandom(fileNegDon,destNegDon,linesNeg,containsSS);
			extractAndWriteRandom(filePosAcc,destPosAcc,linesPos,containsSS);
			extractAndWriteRandom(filePosDon,destPosDon,linesPos,containsSS);
			
			
		}
		else{
			extractAndWriteNonRandom(fileNegAcc,destNegAcc,linesNeg,containsSS);
			extractAndWriteNonRandom(fileNegDon,destNegDon,linesNeg,containsSS);
			extractAndWriteNonRandom(filePosAcc,destPosAcc,linesPos,containsSS);
			extractAndWriteNonRandom(filePosDon,destPosDon,linesPos,containsSS);
		}
	}
	
	public void extractAndWriteRandom(File source, File destiny, int lines, boolean containsSS){
		try{
			List<String> list = new ArrayList<String>();
			List<String> listSS1 = new ArrayList<String>();
			List<String> listSS2 = new ArrayList<String>();
			
			File sourceSS = new File(source.getPath()+".ss");
			File destinySS = new File(destiny.getPath()+".ss");
			
			BufferedReader reader = new BufferedReader(new FileReader(source));
			BufferedWriter writer = new BufferedWriter(new FileWriter(destiny));
			
			BufferedReader readerSS = null;
			BufferedWriter writerSS = null;
			if(containsSS){
				readerSS = new BufferedReader(new FileReader(sourceSS));
				writerSS = new BufferedWriter(new FileWriter(destinySS));
			}
			
			String s = reader.readLine();
			String ss1 = null;
			String ss2 = null;
			
			if(containsSS){
				ss1 = readerSS.readLine();
				ss2 = readerSS.readLine();
			}
			
			while(s!=null){
				if(!s.contains("N")&& !s.contains("n")&& !s.equals("") && !s.equals(" ")){
					s = s.split("\t")[0];
					s = s.trim();
					list.add(s);
					if(containsSS){
						listSS1.add(ss1);
						listSS2.add(ss2);
					}
				}
				s = reader.readLine();
				if(containsSS){
					ss1 = readerSS.readLine();
					ss2 = readerSS.readLine();
				}
			}
			
			boolean [] taken = new boolean[list.size()];
			for(int i=0;i<taken.length;i++)
				taken[i] = false;
			
			
			int counter = 0;
			Random random = new Random();
			while(counter<lines){
				int index = random.nextInt(list.size());
				if(!taken[index]){
					taken[index] = true;
					writer.write(list.get(index)+"\n");
					counter++;
					if(containsSS){
						writerSS.write(listSS1.get(index)+"\n");
						writerSS.write(listSS2.get(index)+"\n");
					}
				}
			}
			
			reader.close();
			writer.close();
			list.clear();
			
			if(containsSS){
				readerSS.close();
				writerSS.close();
				listSS1.clear();
				listSS2.clear();
			}
			
		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
	
	public void extractAndWriteNonRandom(File source, File destiny, int lines, boolean containsSS){
		try{
			
			File sourceSS = new File(source.getPath()+".ss");
			File destinySS = new File(destiny.getPath()+".ss");
			
			BufferedReader reader = new BufferedReader(new FileReader(source));
			BufferedWriter writer = new BufferedWriter(new FileWriter(destiny));
			BufferedReader readerSS = null;
			BufferedWriter writerSS = null;
			if(containsSS){
				readerSS = new BufferedReader(new FileReader(sourceSS));
				writerSS = new BufferedWriter(new FileWriter(destinySS));
			}
			
			String s = reader.readLine();
			String ss1 = null;		//each secondary structure consists of both 
			String ss2 = null;		//primary and secondary structure
			if(containsSS){
				ss1 = readerSS.readLine();
				ss2 = readerSS.readLine();
			}
				
			int counter = 0;
			while(counter<lines){
				while(s.contains("N")||s.contains("n")||s.equals("")){
					s = reader.readLine();
					if(containsSS){
						ss1 = readerSS.readLine();
						ss2 = readerSS.readLine();
					}
				}
				s = s.split("\t")[0];
				s = s.trim();
				counter++;
				writer.write(s+"\n");
				s = reader.readLine();
				if(containsSS){
					writerSS.write(ss1+"\n");
					writerSS.write(ss2+"\n");
					ss1 = readerSS.readLine();
					ss2 = readerSS.readLine();
				}
			}
			
			reader.close(); 
			writer.close();
			if(containsSS){
				readerSS.close();
				writerSS.close();
			}

		}
		catch(Exception exc){
			exc.printStackTrace();
		}
	}
}
