/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.classificationFeatures;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import main.FunSiPJar;
import util.featureSelection.ValPosCombination;


/**
 * This class contains some different utility methods for feature extraction.
 * @author Michiel Van Bel
 *
 */
public class ClassificationFeatureUtils {

	
	public enum CLASSIFICATIONFEATURE_TYPES{POSITIONAL,COMPOSITIONAL,NEAREST_EQUAL,
		READING_FRAME,BRANCH_POINT,SEC_POSITIONAL,SEC_COMPOSITIONAL,SEC_NEAR,POSITIONAL_GENERAL,
		COMPOSITIONAL_GENERAL,NONBINDING,GC,NUC_DISTANCE,TETRAHEDRON,WAVECONTENT};
	
		
	private ClassificationFeatureUtils(){
	}
	
	private static File findCFDirectory(){		
		File result= null;
		result = new File("./util/classificationFeatures/implementations/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("/util/classificationFeatures/implementations/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("/implementations/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("./implementations/");
		if(result.exists()&&result.isDirectory())
			return result;
		return null;
	}
	
	private static File findCFTestDirectory(){
		File result = null;
		result = new File("./util/classificationFeatures/implementations/test/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("/util/classificationFeatures/implementations/test/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("/implementations/test/");
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("./implementations/test/");
		if(result.exists()&&result.isDirectory())
			return result;
		return null;
	}
	
	private static String getJavaPackageNotation(String s){
		//change slashes and backslashes to points
		StringBuffer temp = new StringBuffer();
		for(int i=0;i<s.length();i++){
			char c = s.charAt(i);
			if(c=='\\'||c=='/')
				temp.append(".");
			else
				temp.append(c);
		}
		int inc = 0;
		for(int i=0;i<temp.length();i++)
			if(temp.charAt(i)=='.')
				inc++;
			else
				break;
		String ret = temp.substring(inc,temp.length());
		return ret;
	}
	
	public static ClassificationFeature parseFeatureFromString(String[] split){
		if(FunSiPJar.RUN_FROM_JAR){
			return parseFeatureFromStringJar(split);
		}
		else{
			return parseFeatureFromStringNonJar(split);
		}
	}
	
	public static ClassificationFeature parseFeatureFromStringJar(String[] split){
		if(split==null)
			return null;
		ClassificationFeature feature = null;
		try{
			JarFile jf = new JarFile("FunSiP.jar");
			Enumeration resources = jf.entries();
			List<ClassificationFeature> cfList = new ArrayList<ClassificationFeature>();
			while(resources.hasMoreElements()){
				JarEntry je = (JarEntry)resources.nextElement();
				if((je.getName().startsWith("util/classificationFeatures/implementations/")||
						je.getName().startsWith("util/classificationFeatures/implementations/test/"))&&
						je.getName().endsWith(".class")){
					try{
						String name = je.getName().substring(0,je.getName().length()-6);
						name = getJavaPackageNotation(name);
						Class c = Class.forName(name);
						ClassificationFeature feat = (ClassificationFeature)c.newInstance();
						cfList.add(feat);
					}
					catch(Exception exc){
						//do nothing, is perhaps a malformed classificationFeature
					}
				}
			}
			
			//split from former for-loop for fail-safe concerns. 
			for(int i=0;i<cfList.size();i++){
				ClassificationFeature feat = cfList.get(i);
				try{
					feat.setParameters(split);
					//if no exception is thrown, this one is correct
					return feat;
				}
				catch(Exception exc){
			 
				}
			}	
			
		}
		catch(Exception exc){
			return null;
		}
		return feature;
	}
		
	public static ClassificationFeature parseFeatureFromStringNonJar(String[] split){
		if(split==null)
			return null;
		ClassificationFeature feature = null;
		try{
			File firstDirectory = findCFDirectory();
			File testDirectory = findCFTestDirectory();
			//System.out.println(firstDirectory + " "+testDirectory);
			if(firstDirectory==null || testDirectory==null)
				return null;
			
			List<ClassificationFeature> cfList = new ArrayList<ClassificationFeature>();
			ClassificationFeatureUtils cfu = new ClassificationFeatureUtils();
			FileFilter ff = cfu.new ClassFileFilter();
			File[] impFiles = firstDirectory.listFiles(ff);
			File[] testFiles = testDirectory.listFiles(ff);
			for(File file : impFiles){
				try{
					String toLoad = file.getPath().substring(0,file.getPath().length()-6); //remove the .class at the end
					toLoad = getJavaPackageNotation(toLoad);
					Class c = Class.forName(toLoad);
					ClassificationFeature feat = (ClassificationFeature)c.newInstance();
					cfList.add(feat);
				}
				catch(Exception exc){
					//do nothing, is perhaps a malformed classificationFeature
				}
			}
			for(File file: testFiles){
				try{
					String toLoad = file.getPath().substring(0,file.getPath().length()-6); //remove the .class at the end
					toLoad = getJavaPackageNotation(toLoad);
					Class c = Class.forName(toLoad);
					ClassificationFeature feat = (ClassificationFeature)c.newInstance();
					cfList.add(feat);
				}
				catch(Exception exc){
					//do nothing, is perhaps a malformed classificationFeature
				}
			}
			
			//split from former for-loop for fail-safe concerns. 
			for(int i=0;i<cfList.size();i++){
				ClassificationFeature feat = cfList.get(i);
				try{
					feat.setParameters(split);
					//if no exception is thrown, this one is correct
					return feat;
				}
				catch(Exception exc){
					//System.out.println(exc);
				}
			}			
		}
		catch(Exception exc){
			return null;
		}
		return feature;
	}
	
	/**
	 * This method creates a ClassificationFeature from a string, by using 
	 * the java-reflection to locate the correct ClassificationFeature implementation.
	 * This approach may not be the fastest, but it allows to dynamically add new ClassificationFeatures
	 * to the right directory without having to recompile other files (such as this
	 * ClassificationFeatureUtils - java file). 
	 * @param s The String representation of the ClassificationFeature, as defined in a configuration file
	 * @return The correct ClassificationFeature if it could be located and loaded, null otherwise.
	 */
	public static ClassificationFeature parseFeatureFromString(String s){
		if(s==null)
			return null;
		String[] split = s.split(" ");
		return parseFeatureFromString(split);
	}
	
/*
	public static ClassificationFeature parseFeatureFromString2(String s){
		ClassificationFeature feature = null;
		if(s==null)
			return feature;
		try{
			String[] split = s.split(" ");
			String name = split[0].toUpperCase();
			if(name.equals("P")||name.equals("POSITIONAL")){
				feature = new PositionalClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("PG")||name.equals("POSITIONAL_GENERAL")){
				feature = new PositionalGeneralClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("C")||name.equals("COMPOSITIONAL")){
				feature = new CompositionalClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("CG")||name.equals("COMPOSITIONAL_GENERAL")){
				feature = new CompositionalGeneralClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("RF")||name.equals("READING_FRAME")){
				feature = new RFClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("BP")||name.equals("BRANCHPOINT")){
				feature = new BranchpointClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("GC")){
				feature  = new GcContentClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("PP")||name.equals("PHYSICAL_PROPERTIES")){
				feature  = new PhysicalPropertiesClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("WAVECONTENT")){
				feature = new WaveContentClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("ND")||name.equals("NUC_DISTANCE")){
				feature = new TestNucleotideDistanceClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("NONBINDING")||name.equals("NB")){
				feature = new NonBindingClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("NE")){
				feature = new NearestEqualClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("PSS")){
				feature = new PositionalSecondaryStructureClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("CSS")){
				feature = new CompositionalSecondaryStructureClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("NEARSS")){
				feature = new NearSecondaryStructureClassificationFeature();
				feature.setParameters(split);
			}
			else if(name.equals("TETRAHEDRON")){
				feature = new TetraHedronClassificationFeature();
				feature.setParameters(split);
			}
			else{
				return null;
			}
		}
		catch(Exception exc){
			return null;
		}
		
		return feature;
	}*/
	
	
	public static List<List<ValPosCombination>> divide(List<ClassificationFeature> cfList,List<ValPosCombination> vpcList){
		List<List<ValPosCombination>> res = new ArrayList<List<ValPosCombination>>();
		int[] attributeBoundaries = new int[cfList.size()];
		int current = 1;
		for(int i=0;i<cfList.size();i++){
			ClassificationFeature cf = cfList.get(i);
			int numFeatures = cf.getNumberOfAttributes();
			current+=numFeatures;
			attributeBoundaries[i] = current;
			List<ValPosCombination> list = new ArrayList<ValPosCombination>();
			res.add(list);
		}
		
		for(int i=0;i<vpcList.size();i++){
			ValPosCombination vpc = vpcList.get(i);
			int position = vpc.getPosition();
			for(int j=0;j<cfList.size();j++){
				if(position<attributeBoundaries[j]){
					res.get(j).add(vpc);
					break;
				}
			}
		}
		
		return res;
	}
	
	/**
	 * Selects and returns te maximum value of integers in a list. If the item is
	 * smaller then 0, then 0 is returned.  
	 * @param list The list with integer data
	 * @return The maximum value
	 */
	public static int getMaximum(List<Integer> list){
		int max= Integer.MIN_VALUE;
		for(int i=0;i<list.size();i++){
			int temp = list.get(i);
			if(temp>max)
				max = temp;
		}
		if(max<0)
			max = 0;
		return max;
	}
	
	
	/**
	 * Selects the maximum value of integers in a hashmap
	 */
	/*public static int getMaximum(HashMap<String,Integer> map){
		int max = Integer.MIN_VALUE;
		Iterator<Integer> it = map.values().iterator();
		while(it.hasNext()){
			int t = it.next();
			if(t>max)
				max = t;
		}
		if(max<0)
			max = 0;
		return max;
	}*/
	
	
	/**
	 * Recursively computes all possible n-nucleotides (with n being the length)
	 * @param length The length of the nucleotides
	 * @return The array with all possible combinations
	 */
	public static String[] generateCompositionalTypes(int length){
		if(length<1){
			String [] result = {};
			return result;
		}
		if(length==1){
			String[] result = {"A","C","G","T"};
			return result;
		}
		else{
			String[] append = {"A","C","G","T"};
			String[] temp = generateCompositionalTypes(length-1);
			String[] result = new String[temp.length*4];
			
			for(int j=0;j<append.length;j++){
				for(int i=0;i<temp.length;i++){
					result[i*4+j]=temp[i]+append[j];
				}
			}
			return result;
		}
	}
	
	/**
	 * Generates all possible combinations, according to the provided alphabet and the 
	 * required length of the combinations.
	 * @param length The length of the combinations
	 * @param alphabet The alphabet used to produce the combinations 
	 * @return The list of possible combinations
	 */
	public static String[] generateGeneralCompositionalTypes(int length,char[] alphabet){
		String[] newAlphabet = new String[alphabet.length];
		for(int i=0;i<alphabet.length;i++)
			newAlphabet[i] = ""+alphabet[i];
		return generateGeneralCompositionalTypes(length,newAlphabet);
	}
	
	
	/**
	 * Generates all possible combinations, according to the provided alphabet and the 
	 * required length of the combinations.
	 * @param length The length of the combinations
	 * @param alphabet The alphabet used to produce the combinations 
	 * @return The list of possible combinations
	 */
	public static String[] generateGeneralCompositionalTypes(int length, String[] alphabet){
		if(length<1){
			String[] result = {};
			return result;
		}
		else if(length==1){
			return alphabet;
		}
		else{
			String[] temp = generateGeneralCompositionalTypes(length-1,alphabet);
			String[] result = new String[temp.length*alphabet.length];
			for(int j=0;j<alphabet.length;j++){
				for(int i=0;i<temp.length;i++){
					result[i*alphabet.length+j]=temp[i]+alphabet[j];
				}
			}
			return result;
		}
	}
	
	
	/**
	 * Recusrively computes all possible n-nucleotides that are 
	 * reduced to the set of purines and pyrimidines.
	 * @param length The length of the combinations
	 * @return The array with all possible combinations
	 */
	public static String[] generateReducedCompositionalTypes(int length){
		if(length<1){
			String[] result = {};
			return result;
		}
		if(length==1){
			String[] result = {"R","Y"};
			return result;
		}
		else{
			String[] append = {"R","Y"};
			String[] temp = generateReducedCompositionalTypes(length-1);
			String[] result = new String[temp.length*2];
			for(int j=0;j<append.length;j++){
				for(int i=0;i<temp.length;i++){
					result[i*2+j] = temp[i]+append[j];
				}
			}
			return result;
		}
	}
	
	/**
	 * Recursively computes all possible combinations of stems and loops 
	 * of length n 
	 * 3 possible characters : ( ) .
	 * @param length The length of the combinations
	 * @return The array with all the possible combinations
	 */
	public static String[] generateFullSecStructCompositionalTypes(int length){
		if(length<1){
			String [] result = {};
			return result;
		}
		if(length==1){
			String [] result = {"(",")","."};
			return result;
		}
		else{
			String[] append = {"(",")","."};
			String[] temp = generateFullSecStructCompositionalTypes(length-1);
			String[] result = new String[temp.length*3];
			for(int j=0;j<append.length;j++){
				for(int i=0;i<temp.length;i++){
					result[i*3+j]=temp[i]+append[j];
				}
			}
			return result;
		}
	}
	
	/**
	 * Recursively computes all possible combinations of stems and loops 
	 * of length n 
	 * 2 possible characters : | . 
	 * @param length The length of the combinations
	 * @return The array with all the possible combinations
	 */
	public static String[] generateReducedSecStructCompositionalTypes(int length){
		if(length<1){
			String [] result = {};
			return result;
		}
		if(length==1){
			String [] result = {"|","."};
			return result;
		}
		else{
			String[] append = {"|","."};
			String[] temp = generateFullSecStructCompositionalTypes(length-1);
			String[] result = new String[temp.length*2];
			for(int j=0;j<append.length;j++){
				for(int i=0;i<temp.length;i++){
					result[i*2+j]=temp[i]+append[j];
				}
			}
			return result;
		}
	}
	
	/**
	 * Replace all ( and ) by | in the secondary structure string
	 * @param t original string
	 * @return The reformed string
	 */
	public static String convertFullSecToReducedSec(String t){
		StringBuffer buffer = new StringBuffer();
		for(int i=0;i<t.length();i++){
			switch(t.charAt(i)){
			case '(' : case ')' : buffer.append('|'); break;
			case '.' : buffer.append('.'); break;
			default : buffer.append('.'); break;
			}
		}
		
		return buffer.toString();
		
	}
	
	
	
	/**
	 * Adds all items from the featurevector (represented by an array) to the 
	 * list with items.
	 * @param list The existing list
	 * @param featureVec The featurevector
	 * @return The boolean value indicates whether or not the merge did go flawless.
	 */
	public static boolean mergeListFeatureVector(List<Integer> list,int[] featureVec){
		if(list==null || featureVec==null) 
			return false;
		for(int i=0;i<featureVec.length;i++){
			list.add(featureVec[i]);
		}
		return true;
	}
	
	public static void main(String[] args){
		
		String[] alphabet = {"S","W"};
		String [] res = generateGeneralCompositionalTypes(5,alphabet);
		for(String s : res)
			System.out.println(s);
		/*long temp1 = System.nanoTime();
		generateCompositionalTypes(6);
		long temp2 = System.nanoTime();
		System.out.println("tijd lengte 6 : "+(temp2-temp1));
		int[] temp = new int[10];
		for(int i=0;i<temp.length;i++)
			System.out.print(temp[i]+" ");*/
	}
	
	private class ClassFileFilter implements FileFilter{
		public ClassFileFilter(){
		}
		public boolean accept (File file){
			if(!file.getName().endsWith(".class"))
				return false;
			else 
				return true;
		}
	}
	
}
