package util.classificationFeatures.implementations.test;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import main.FunSiPFeatureExplorer.POSSIBLE_COMBINATION_TYPES;
import util.classificationFeatures.AttributeDescription;
import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureDrawing;
import util.classificationFeatures.ClassificationFeatureUtils;
import util.classificationFeatures.FeatureExtractionException;
import util.classificationFeatures.IncorrectClassificationFeatureException;
import util.conversions.Conversion;

public class ManualSelectedCombinationsClassificationFeature implements ClassificationFeature{

	private String fileName = null;
	private List<ClassificationFeature> features = null;
	private List<CombinationIndicator> indicators = null ;
	private POSSIBLE_COMBINATION_TYPES type = POSSIBLE_COMBINATION_TYPES.NOT_PRESENT_COMBINATIONS;
	
	private class CombinationIndicator{
		public int feature_1 = -1;
		public int feature_2 = -1;
		public int location_1 = -1;
		public int location_2 = -1;
		
		public CombinationIndicator(int feat1,int loc1,int feat2,int loc2){
			feature_1 = feat1;
			location_1 = loc1;
			feature_2 = feat2;
			location_2 = loc2;
		}
		
	}
	
	
	public ManualSelectedCombinationsClassificationFeature(){
		features = new ArrayList<ClassificationFeature>();
		indicators = new ArrayList<CombinationIndicator>();
	}
	
	/**
	 * This method asks for the up- and downstreamranges of the feature
	 * @return The up/down ranges
	 */
	public int[] getUpDownRanges(){
		int [] res = {-1,-1};
		return res;
	}
	
	/**
	 * This method asks the classificationfeature for its conversion-type.
	 * @return The used conversion type. Null of not appropriate.
	 */
	public Conversion getConversion(){
		return null;
	}
	
	
	/**
	 * This method sets the new upstream and downstream ranges  
	 * @param up The new upstream range
	 * @param down The new downstream range
	 */
	public void setUpDownRanges(int up, int down){
	}

	/**
	 * Asks whether the computer needs to precompute compositional types.
	 * When a conversion is taken with a large alphabet and/or a large length,
	 * this computation can take WAAAY to long (and take too much memory as well).
	 * Therefore, prior to this operation, this flag should be checked.
	 * @return Whether or not to supply compositional types of certain length.
	 */
	public boolean needCompositionalTypes(){
		return false;
	}
	
	/**
	 * This method extracts the features from a sequence (relative to the splicesite)
	 * and returns those features in a list of double-values (standardized on double
	 * values because of its generic but still small-sized nature).
	 * @param sequences The sequence from which the features should be extracted.
	 * @param splicesite The location of the splicesite (pseudo or real)
	 * @param types The compositional types (if requested, otherwise null).
	 * @param conversions A list with all the conversion types
	 * @param conversion_sequences  An array with all the converted sequences.
	 * @return A list with the features
	 * @throws FeatureExtractionException Thrown when something goes wrong when extracting the features.
	 */
	public List<Double> extractFeatures(String sequences[], int splicesite,String[] types, List<Conversion> conversions,String[]conversion_sequences) throws FeatureExtractionException{
		if(sequences==null||sequences.length<1||sequences[0]==null)
			throw new FeatureExtractionException("Stringsequence is null or could not be retrieved.");
		List<Double> result = new ArrayList<Double>();
		
		//first get all the results of the features
		List<List<Double>> temporary_results = new ArrayList<List<Double>>();
		for(int i=0;i<features.size();i++){
			ClassificationFeature feat = features.get(i);
			String[] new_types = null;
			if(feat.needCompositionalTypes()){
				 new_types = ClassificationFeatureUtils.generateCompositionalTypes(feat.getLength()); 
			}
			List<Double>f = feat.extractFeatures(sequences, splicesite, new_types, conversions, conversion_sequences);
			temporary_results.add(f);
		}
		
		//then, select only the correct ones, and - according to the type -label the results.
		for(int i=0;i<indicators.size();i++){
			CombinationIndicator indicator = indicators.get(i);
			List<Double> featList1 = temporary_results.get(indicator.feature_1);
			List<Double> featList2 = temporary_results.get(indicator.feature_2);
			
			Double feat1 = featList1.get(indicator.location_1);
			Double feat2 = featList2.get(indicator.location_2);
			
			switch(type){
			case NOT_PRESENT_COMBINATIONS :
				if(feat1==0.0 && feat2==0.0)
					result.add(1.0);
				else
					result.add(0.0);
				break;
			case PRESENT_COMBINATIONS :
				if(feat1>0 && feat2>0)
					result.add(1.0);
				else
					result.add(0.0);
				break;
			default : break;	
			}
		}
		
		return result;
	}
	

	
	/**
	 * Returns the string representation of the classificationfeature,
	 * but with all possible spaces and tabs replaced by underscores.
	 * @return Alternative string representation
	 */
	public String toStringUnderscore(){
		return "MANUAL_COMBINATIONS_"+fileName;
	}
	
	
	/**
	 * String representation of the classificationFeature
	 * @return String representation of the classificationfeature
	 */
	public String toString(){
		return "MANUAL_COMBINATIONS "+fileName;
	}
	
	/**
	 * String representation of the classificationFeature that contains
	 * the core elements, and discards the upstream/downstream range
	 * elements.
	 * @return Shortened version of the string representation of the classificationfeature
	 */
	public String toNoRangeString(){
		return "MANUAL_COMBINATIONS "+fileName;
	}
	
	/**
	 * Returns a standardized description of the location that is indicated.
	 * e.g. 5 bp upstream of splicesite. In case the location is outside the range
	 * of the feature, an error string is returned.
	 * @param loc The location
	 * @return The description
	 */
	public AttributeDescription getLocationDescription(int loc){
		return null;
	}
	
	/**
	 * Returns the number of attributes that are created by this classificationfeature,
	 * according to the internal parameters (e.g. up- and downstreamranges). When the 
	 * internal parameters are not set, or they are in an incorrect state, this method will
	 * return -1 as value
	 * @return The number of attributes created by this classificationfeature
	 */
	public int getNumberOfAttributes(){
		return indicators.size();
	}
	
	
	/**
	 * This method sets the necessary parameters for the classificationfeature. 
	 * The parameters are all generically passed as an array of strings, which of course
	 * need to be decoded by the classificationfeature itself.
	 * @param args The array with the parameters in string format
	 * @throws IncorrectClassificationFeatureException Can be thrown when the parameters
	 * are incorrect (outside possible ranges) and/or the parameters could not be parsed.
	 */
	public void setParameters(String[] args) throws IncorrectClassificationFeatureException{
		if(args.length!=2){
			throw new IncorrectClassificationFeatureException("Incorrect number of parameters for MANUAL_COMBINATION feature");
		}
		String name = args[0].toUpperCase();
		if(!name.equals("MANUAL_COMBINATION")){
			throw new IncorrectClassificationFeatureException("Incorrect naming for MANUAL_COMBINATION feature");
		}
		try{
			fileName = args[1];
			readFileContent();
		}
		catch(Exception exc){
			//exc.printStackTrace();
			throw new IncorrectClassificationFeatureException("Unable to load parameterfile for MANUAL_COMBINATION feature");
		}
	}
	
	private void readFileContent()throws Exception{
		BufferedReader reader = new BufferedReader(new FileReader(new File(fileName)));
		String s = reader.readLine();
		while(s!=null){
			String[] split = s.split("\t+");
			if(split[0].equals("TYPE")){
				if(split[1].equals(POSSIBLE_COMBINATION_TYPES.NOT_PRESENT_COMBINATIONS.toString())){
					type = POSSIBLE_COMBINATION_TYPES.NOT_PRESENT_COMBINATIONS;
				}
				else if(split[1].equals(POSSIBLE_COMBINATION_TYPES.PRESENT_COMBINATIONS.toString())){
					type = POSSIBLE_COMBINATION_TYPES.PRESENT_COMBINATIONS;
				}
				else
					throw new Exception("Unknown types found during parsing of parameterfile Manual_selected feature");
			}
			else if(split[0].equals("CLASSIFICATION_FEATURE")){
				ClassificationFeature feat = ClassificationFeatureUtils.parseFeatureFromString(split[1]);
				if(feat!=null)
					features.add(feat);
				else
					throw new Exception("Impossible to parse correct feature during parsing of parameterfile Manual_selected feature");
			}
			else if(split[0].equals("COMBINATION")){
				String[] temp = split[1].split(" +");
				int feature_1 = Integer.parseInt(temp[0]);
				int position_1 = Integer.parseInt(temp[1]);
				int feature_2 = Integer.parseInt(temp[2]);
				int position_2 = Integer.parseInt(temp[3]);
				CombinationIndicator ind = new CombinationIndicator(feature_1,position_1,feature_2,position_2);
				indicators.add(ind);
			}
			s = reader.readLine();
		}
		reader.close();
	}
	
	/**
	 * Because all classificationfeatures need different algorithms to draw them,
	 * according to the specifications, these utilities are also moved to the 
	 * implementations of the ClassificationFeatures. However, as not to clutter them,
	 * we have chosen to place this data in a seperate interface. The programmer thus
	 * has a choice of using one or two classes to implement both interfaces.
	 * 
	 * @return The object which can draw the classification features.
	 */
	public ClassificationFeatureDrawing getClassificationFeatureDrawing(){
		return null;
	}

	
	/**
	 * @return The downstream_range for this positional feature
	 */
	public int getDownstream_range() {
		return -1;
	}

	/**
	 * @return The length of the positional feature (most used lengths are 1,2 or 3)
	 */
	public int getLength() {
		return -1;
	}

	/**
	 * @return The upstream_range for this positional feature
	 */
	public int getUpstream_range() {
		return -1;
	}


	public void setDownstream_range(int downstream_range) {
	}


	public void setLength(int length) {
	}


	public void setUpstream_range(int upstream_range) {
	}
	
	/**
	 * Makes a copy of this particular classificationfeature. 
	 * The cloneable interface cannot be used directly because it is only applicable
	 * on classes, not interfaces.
	 * @return A copy of the classificationfeature
	 */
	public ClassificationFeature make_copy(){
		ClassificationFeature res = null;
		try{
			res = (ClassificationFeature)this.clone();
		}
		catch(Exception exc){}
		return res;
	}
	
	
}
