/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.conversions;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;

import main.FunSiPJar;
import util.conversions.implementations.DNA_AA_Conversion;
import util.conversions.implementations.DNA_MK_Conversion;
import util.conversions.implementations.DNA_RY_Conversion;
import util.conversions.implementations.DNA_SW_Conversion;
import util.conversions.implementations.TEST_Conversion;

public class ConversionUtils {
	
	private ConversionUtils(){
	}

	
	/**
	 * This method constructs the actual conversion and translation
	 * objects.
	 * @param split The array of strings that contains the description of the 
	 * conversion object
	 * @return The conversion object.
	 */
	public static Conversion parseConversionFromString(String[] split){
		if(FunSiPJar.RUN_FROM_JAR){
			return parseConversionFromStringJar(split);
		}
		else{
			return parseConversionFromStringNonJar(split);
		}
			
	}
	
	private static String getJavaPackageNotation(String s){
		//change slashes and backslashes to points
		StringBuffer temp = new StringBuffer();
		for(int i=0;i<s.length();i++){
			char c = s.charAt(i);
			if(c=='\\'||c=='/')
				temp.append(".");
			else
				temp.append(c);
		}
		int inc = 0;
		for(int i=0;i<temp.length();i++)
			if(temp.charAt(i)=='.')
				inc++;
			else
				break;
		String ret = temp.substring(inc,temp.length());
		return ret;
	}
	
	
	public static Conversion parseConversionFromStringJar(String[] split){
		Conversion result = null;
		try{
			JarFile jf = new JarFile("FunSiP.jar");
			Enumeration resources = jf.entries();
			List<Conversion> conversions = new ArrayList<Conversion>();
			while(resources.hasMoreElements()){
				JarEntry je = (JarEntry)resources.nextElement();
				if((je.getName().startsWith("util/conversions/implementations/")||
						je.getName().startsWith("util/conversions/implementations/test/"))&&
						je.getName().endsWith(".class")){
					try{
						String name = je.getName().substring(0,je.getName().length()-6);
						name = getJavaPackageNotation(name);
						Class c = Class.forName(name);
						Conversion conv = (Conversion)c.newInstance();
						conversions.add(conv);
					}
					catch(Exception exc){
						
					}
				}
			}
			boolean accepted = false;
			//split from former for-loop for fail-safe concerns. 
			for(int i=0;i<conversions.size();i++){
				Conversion conv = conversions.get(i);
				accepted = conv.setOptions(split);
				if(accepted)
					return conv;
			}
			if(!accepted){
				return null;
			}
		}
		catch(Exception exc){
			return null;
		}
		return result;
	}
	
	
	/**
	 * This method constructs the actual conversion and translation
	 * objects. 
	 * @param split The array of string that contains the description of the 
	 * conversion object
	 * @return The conversion object.
	 */
	public static Conversion parseConversionFromStringNonJar(String[] split){
		
		Conversion result = null;
		//String[] split = s.split(" ");
		
		try{
			File conversionsDir = findConversionsDir();
			if(conversionsDir == null)
				return null;
			List<Conversion> conversions = new ArrayList<Conversion>();
			ConversionUtils cu = new ConversionUtils();
			FileFilter ff = cu.new ClassFileFilter();
			File[] files = conversionsDir.listFiles(ff);
			for(File file : files){
				try{
					String toLoad = file.getPath().substring(0,file.getPath().length()-6); //remove the .class at the end
					//change slashes and backslashes to points
					StringBuffer temp = new StringBuffer();
					for(int i=0;i<toLoad.length();i++){
						char c = toLoad.charAt(i);
						if(c=='\\'||c=='/')
							temp.append(".");
						else
							temp.append(c);
					}
					toLoad = temp.substring(2,temp.length());
					Class c = Class.forName(toLoad);
					Conversion conv = (Conversion)c.newInstance();
					conversions.add(conv);
				}
				catch(Exception exc){
				}
			}
			
			boolean accepted = false;
			//split from former for-loop for fail-safe concerns. 
			for(int i=0;i<conversions.size();i++){
				Conversion conv = conversions.get(i);
				accepted = conv.setOptions(split);
				if(accepted)
					return conv;
			}
			if(!accepted){
				return null;
			}
		}
		catch(Exception exc){
			return null;
		}
		return result;
	}
	
	
	private static File findConversionsDir() throws IOException{
		File result = null;
		String t = ConversionUtils.class.getName();
		int lastPoint = t.lastIndexOf('.');
		t = t.substring(0, lastPoint);
		String dirName = "./"+t+"/implementations/";
		result = new File(dirName);
		if(result.exists()&&result.isDirectory())
			return result;
		result = new File("./util/conversions/implementations/");
		return result;
	}
	
	/**
	 * This method constructs the actual conversion and translation
	 * objects.
	 * @param s The string that contains the description of the 
	 * conversion object
	 * @return The conversion object.
	 */
	public static Conversion parseConversionFromString2(String s){
		
		Conversion result = null;
		try{
			String[] split = s.split(" ");
			String name = split[0];
			if(name.equals("DNA_RY")){				//puRine && pYrimidine
				result = new DNA_RY_Conversion();
			}
			else if(name.equals("DNA_MK")){			//aMino && Keto
				result = new DNA_MK_Conversion();
			}
			else if(name.equals("DNA_SW")){			//Strong && Weak
				result = new DNA_SW_Conversion();
			}
			else if(name.equals("DNA_AA")){
				result = new DNA_AA_Conversion();	//to amino acids
			}
			else if(name.equals("TEST_CONVERSION")){
				result = new TEST_Conversion();
			}
			
			if(result!=null){
				boolean pos = result.setOptions(split);
				if(!pos)
					return null;
			}
		}
		catch(Exception exc){
			return null;
		}
		return result;
	}
	
	private class ClassFileFilter implements FileFilter{
		public ClassFileFilter(){
		}
		public boolean accept (File file){
			if(!file.getName().endsWith(".class"))
				return false;
			else 
				return true;
		}
	}

}

