/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.conversions.implementations;

import util.conversions.Conversion;

public class DNA_RY_Conversion implements Conversion{

	public final static String name = "DNA_PYRIMIDINE_PURINE_CONVERSION";
	
	public String getName(){
		return name;
	}
	
	public String makeConversion(String sequence){
		sequence = sequence.toUpperCase();
		char[] res = new char[sequence.length()];
		
		for(int i=0;i<sequence.length();i++){
			char t = sequence.charAt(i);
			switch(t){
			case 'A' : case 'G' : res[i] = 'R'; break;
			case 'C' : case 'T' : case 'U' : res[i] = 'Y'; break;
			default : res[i] = 'N'; break;
			}
		}
		return new String(res);
	}
	
	
	public boolean setOptions(String[] options){ 
		if(options[0].toUpperCase().equals("DNA_RY"))
			return true;
		else
			return false;
	}
	
	/**
	 * Returns the parameter name of the conversion. This is usefull for
	 * compatibility with gui's and other programs which rely on the
	 * parameter for identification.
	 * @return String with parametername (unique).
	 */
	public String getParamName(){
		return "DNA_RY";
	}
	
	
	public int getNumberOfDifferentCharacters(){
		return 2;
	}
	

	public char[] getDifferentCharacters(){
		char[] result = {'R','Y'};
		return result;
	}
}
