/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.featureSelection;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import util.SortingAlgorithms;
import weka.attributeSelection.AttributeEvaluator;
import weka.core.Instances;

public class AbstractAttributeEvaluation {

	private Logger logger = null;
	private AttributeEvaluator ae = null;
	private Instances instances = null;
	
	public AbstractAttributeEvaluation(Logger logger, Instances instances,AttributeEvaluator eval){
		this.logger = logger;
		this.instances = instances;
		this.ae = eval;
	}
	
	public void buildEvaluator(){
		try{
			ae.buildEvaluator(instances);
		}
		catch(Exception exc){
			logger.error("Error while building the feature selection evaluator");
			logger.error(exc);
		}
	}
	
	
	public List<Double> returnAllEvaluations(){
		List<Double> result = new ArrayList<Double>();
		try{
			
			for(int i=0;i<instances.numAttributes();i++){
				Double d = ae.evaluateAttribute(i);
				result.add(d);
			}
		}
		catch(Exception exc){
			logger.error("Error while evaluating the attributes");
			logger.error(exc);
		}
		return result;
	}
	
	/**
	 * Returns a list with combinations of values and their original positions within
	 * the list of evaluations. This way we can extract the best values and see 
	 * to which feature the selected attribute belongs.
	 * @return A list of values/positions
	 */
	public List<ValPosCombination> returnSortedEvaluation(){
		List<ValPosCombination> result = new ArrayList<ValPosCombination>();
		List<Double> temp = returnAllEvaluations();
		int[] backtrack = new int[temp.size()];
		double[] data_copy = new double[temp.size()]; 
		for(int i=0;i<temp.size();i++){
			backtrack[i]=i;
			data_copy[i] = temp.get(i);
		}
		backtrack = SortingAlgorithms.mergeSortDual(data_copy,backtrack);
		
		for(int i=0;i<backtrack.length;i++){
			ValPosCombination combo = new ValPosCombination(data_copy[i],backtrack[i]);
			result.add(combo);
		}
		
		return result;
	}
	
	public List<ValPosCombination> returnUnsortedEvaluation(){
		List<ValPosCombination> result = new ArrayList<ValPosCombination>();
		List<Double> temp = returnAllEvaluations();
		int[] backtrack = new int[temp.size()];
		double[] data_copy = new double[temp.size()]; 
		for(int i=0;i<temp.size();i++){
			backtrack[i]=i;
			data_copy[i] = temp.get(i);
		}
		for(int i=0;i<backtrack.length;i++){
			ValPosCombination combo = new ValPosCombination(data_copy[i],backtrack[i]);
			result.add(combo);
		}	
		return result;
	}
}
