/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.featureSelection;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.ArrayList;
import java.util.List;

import util.classificationFeatures.ClassificationFeature;
import util.classificationFeatures.ClassificationFeatureUtils;


/**
 * This class provides the means to investigate the possible
 * regulatory features indicated by the features that are selected
 * by feature selection methods.
 * @author Michiel Van Bel
 */
public class FeatureSelectionSupport {

	
	/*
	 * It should be possible to call this class from
	 * the commandline or from a script. 
	 * 
	 */
	public static void main(String[] args){
		if(args.length==4)
			new FeatureSelectionSupport(args[0],args[1],args[2],args[3]);
		else{
			System.err.println("Incorrect number of parameters. Usage : ");
			System.err.println("java util.featureSelection.FeatureSelectionSupport <feature_file> <outputpath> <positive_data> <negative_data>");
			System.exit(1);
		}
	}
	
	public FeatureSelectionSupport(String fsFileName,String outputPath,String posDataFileName, String negDataFileName){
		try{
			//first a couple of tests to define whether the input is even remotely possible
			if(fsFileName==null || outputPath==null || posDataFileName==null || negDataFileName==null){
				System.err.println("FeatureSelectionSupport constructor : parameters cannot be null");
				return;
			}
			{
				File fsFile = new File(fsFileName);
				File output = new File(outputPath);
				File posData = new File(posDataFileName);
				File negData = new File(negDataFileName);
				if(!(fsFile.exists()&& output.exists() && posData.exists() && negData.exists())){
					System.err.println("FeatureSelectionSupport constructor : one of the parameters does not exist");
					return;
				}
			}
			
			List<ValPosCombination> featureValues = new ArrayList<ValPosCombination>();
			ClassificationData data = readFeatureFile(fsFileName,featureValues);
			if(data!=null){
				//int[] classification_sites = findClassificationSites(data.features);
			}
			
				
		}
		catch(Exception exc){
			exc.printStackTrace();
			return; //screw output
		}
		
	}
	
	
	public int[] findClassificationSites(List<ClassificationFeature> list){
		int[] result = new int[list.size()+1];
		result[0] = 0;
		
		return result;
	}
	
	public ClassificationData readFeatureFile(String fileName,List<ValPosCombination> result){
		ClassificationData data = new ClassificationData();
		try{
			BufferedReader reader = new BufferedReader(new FileReader(new File(fileName)));
			String s = reader.readLine();
			//read header first
			while(s!=null && !s.equals("END")){
				String[] split = s.split("\t+");
				if(split[0].equals("CLASSIFICATION_FEATURE")){
					ClassificationFeature feature = ClassificationFeatureUtils.parseFeatureFromString(split[1]);
					data.features.add(feature);
				}
				else if(split[0].equals("SPLICESITE")){
					data.splicesite = Integer.parseInt(split[1]);
				}
				s = reader.readLine();
			}
			
			//now read the data
			int counter = 0;
			while(s!=null){
				String[] split = s.split("\t+");
				if(split.length==2){
					int pos 	= Integer.parseInt(split[0]);
					double val 	= Double.parseDouble(split[1]);
					ValPosCombination com = new ValPosCombination(val,pos);
					result.add(com);
				}
				else{
					int pos = counter;
					double val = Double.parseDouble(split[0]);
					ValPosCombination com = new ValPosCombination(val,pos);
					result.add(com);
					counter++;
				}
				
				s = reader.readLine();
			}
		}
		catch(Exception exc){
			exc.printStackTrace();
			System.exit(1);
		}
		return data;
	}
	
	private class ClassificationData{
		public List<ClassificationFeature> features = new ArrayList<ClassificationFeature>();
		public int splicesite = 0;
		
	}
	
}
