/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import org.apache.log4j.Logger;

import util.Options;

public class ArffFileSupport {

	private Options options = null;
	private Logger logger = null;
	
	public ArffFileSupport(Options options, Logger logger){
		this.options = options;
		this.logger = logger;
	}
	
	
	public void mergeFeatureFiles(boolean isDonor,File positiveFile, File negativeFile){
		//preparing all the variabels, with differences between donors/acceptors
		String fileName = null;
		String zipEntryName = null;
		if(isDonor){
			fileName = options.OUTPUTDIRECTORY+options.outputFileFeaturesDonor+".zip";
			zipEntryName = options.outputFileFeaturesDonor;
		}
		else{
			fileName= options.OUTPUTDIRECTORY+options.outputFileFeaturesAcceptor+".zip";
			zipEntryName = options.outputFileFeaturesAcceptor;
		}
		File outputFile = new File(fileName);
		if(outputFile.exists()) {
			logger.error("File for feature output allready exists. Skipping...");
			return;
		}
		try{
			//create the input and output directives for the positive data
			ZipInputStream zis = new ZipInputStream(new FileInputStream(positiveFile));
			zis.getNextEntry();
			BufferedReader reader = new BufferedReader(new InputStreamReader(zis));
			
			ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(outputFile,true));
			ZipEntry entry = new ZipEntry(zipEntryName);
			zos.putNextEntry(entry);
			BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(zos));
			
			String line = reader.readLine();
			int numFeat = getNumFeaturesArffFile(line);
			//write the header
			String header = generateArffHeader(isDonor,true,numFeat);
			writer.write(header);
			logger.info("STEP2 		: header written");
			
			logger.info("STEP2 		: Writing positive data");	
			//write all the positive data
			while(line!=null){
				writer.write(line+"1\n");
				line = reader.readLine();
			}
			
			
			//create the input directives for the negative data (output is same as for positive data)
			reader.close();
			zis.close();
			
			zis = new ZipInputStream(new FileInputStream(negativeFile));
			zis.getNextEntry();
			reader = new BufferedReader(new InputStreamReader(zis));
			
			//write all the negative data
			logger.info("STEP2 		: Writing negative data");
			line = reader.readLine();
			while(line!=null){
				writer.write(line+"0\n");
				line = reader.readLine();
			}
			
			
			//close all files etc.
			reader.close();
			zis.close();
			writer.close();
			zos.close();
			
			//deleting the temporary files
			positiveFile.delete();
			negativeFile.delete();
		}
		catch(Exception exc){
			logger.error(exc);
		}	
	}
	
	public int getNumFeaturesArffFile(String line){
		int result = 0;
		for(int i=0;i<line.length();i++)
			if(line.charAt(i)==',')
				result++;
		return result;
	}
	
	public String generateArffHeader(boolean isDonor,boolean addClassIndex,int numFeat){
		StringBuffer result = new StringBuffer();
		if(isDonor){
			result.append("@relation donorData\n");
		}
		else {
			result.append("@relation acceptorData\n");
		}
		
		int sum = 0;
		for(int i=0;i<numFeat;i++)
			result.append("@attribute attrib_seq_"+ ++sum+" numeric\n");
		if(addClassIndex){
			result.append("@attribute Class {0,1}\n");
		}
		result.append("@data\n");
		return new String(result);
	}
	
	public String convertArffFileToNtrFile(boolean isDonor){
		String returnVal = null;
		try{
			//creating the local variables,
			//initializing them according to the fact if its a donor or not. 
			File arffFile = null;
			File ntrFile = null;
			if(isDonor){
				arffFile = new File(options.OUTPUTDIRECTORY+options.outputFileFeaturesDonor+".zip");
				ntrFile = new File(options.OUTPUTDIRECTORY+options.outputFileFeaturesDonor+".ntr");
				returnVal = options.OUTPUTDIRECTORY+options.outputFileFeaturesDonor+".ntr";
			}
			else{
				arffFile = new File(options.OUTPUTDIRECTORY+options.outputFileFeaturesAcceptor+".zip");
				ntrFile = new File(options.OUTPUTDIRECTORY+options.outputFileFeaturesAcceptor+".ntr");
				returnVal = options.OUTPUTDIRECTORY+options.outputFileFeaturesAcceptor+".ntr";
			}
			ZipInputStream zis = new ZipInputStream(new FileInputStream(arffFile));
			zis.getNextEntry();
			InputStreamReader isReader = new InputStreamReader(zis);
			BufferedReader reader = new BufferedReader(isReader);
			
			BufferedWriter writer = new BufferedWriter(new FileWriter(ntrFile));
			
			String s = reader.readLine();
			while(s!=null&& !s.equals("@data")){
				s = reader.readLine();
			}
			s = reader.readLine();	
			while(s!=null){
				s = convertArffToNtr(s);
				writer.write(s+"\n");
				 s = reader.readLine();
			}
			writer.close();
			reader.close();
		}
		catch(Exception exc){
			logger.error(exc);
		}
		return returnVal;
	}
	
	public String convertArffToNtr(String s){
		StringBuffer sb = new StringBuffer();
		char temp = s.charAt(s.length()-1);
		if(temp=='1')
			sb.append(temp+" ");
		else
			sb.append("-1 ");
		int counter = 0;
		StringBuffer te = new StringBuffer();
		for(int i=0;i<s.length()-2;i++){
			char c = s.charAt(i);
			if(c!=','){
				te.append(c);
			}
			else{
				counter++;
				String qw = new String(te);
				if(!qw.equals("0")&&!qw.equals("0.0")){
					if(!qw.endsWith(".0"))
						sb.append(""+counter+":"+qw+" ");
					else
						sb.append(""+counter+":"+qw.substring(0,qw.length()-2)+" ");
				}
				te = new StringBuffer();
			}
		}
		counter++;
		sb.append(""+counter+":"+te+" ");
		te = new StringBuffer();
		
		return new String(sb);
	}
	
	public String convertArffToNtrOld(String s){
		StringBuffer sb = new StringBuffer();
		char temp = s.charAt(s.length()-1);
		if(temp=='1')
			sb.append(temp+" ");
		else
			sb.append("-1 ");
		int counter = 0;
		StringBuffer te = new StringBuffer();
		for(int i=0;i<s.length()-2;i++){
			char c = s.charAt(i);
			if(c!=','){
				te.append(c);
			}
			else{
				counter++;
				sb.append(""+counter+":"+te+" ");
				te = new StringBuffer();
			}
		}
		counter++;
		sb.append(""+counter+":"+te+" ");
		te = new StringBuffer();
		
		return new String(sb);
	}
}
