/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.List;
import java.util.Random;
import java.util.SortedSet;
import java.util.TreeSet;

import org.apache.log4j.Logger;

import util.Options;
import util.RocCurveData;
import util.classificationFeatures.ClassificationFeature;
import util.results.CrossValidationResult;


/**
 * Util class which contains some necessary but rather specific I/O methods
 * (mostly file I/O) for the JASPr code. As such, not a lot of these methods 
 * can be salvaged for use in other programs.
 * @author Michiel Van Bel
 */
public class FileUtils {
	
	//can be used for testing purposes if necessary.
	public static void main(String [] args){
			
	}
	
	
	
	/**
	 * This method merges the results from 4 different files, by sorting them 
	 * on the number which is in the first column (it is assumed that the files
	 * are thus well-formatted). The files are supposed to be the end-result of
	 * the prediction algorithm, but it is required that only one file is delivered.
	 * @param f_d The file with forward donor data results
	 * @param f_a The file with forward acceptor data results
	 * @param r_d The file with reverse donor data results
	 * @param r_a The file with reverse acceptor data results
	 * @param name The name of the resultin merged file
	 * 
	 * 
	 * TODO : pretty specific and dependent code, better would be and array/list of files,
	 * a name for the resulting file, an index for designating which column to sort on (
	 * in the files to be merged) and the split-character (most likely tab or space).
	 */
	public static void mergeAndDeleteResults(File f_d, File f_a, File r_d, File r_a, String name){
		if(f_d==null || f_a==null ||r_d == null|| r_a ==null ||name ==null){
			System.err.println("Cannot merge results : nullpointer detected");
			return;
		}
		
		try{
			BufferedReader f_d_r = new BufferedReader(new FileReader(f_d));	
			BufferedReader f_a_r = new BufferedReader(new FileReader(f_a));
			BufferedReader r_d_r = new BufferedReader(new FileReader(r_d));
			BufferedReader r_a_r = new BufferedReader(new FileReader(r_a));
			BufferedWriter writer = new BufferedWriter(new FileWriter(name));
			
			String f_d_line = f_d_r.readLine();
			String f_a_line = f_a_r.readLine();
			String r_d_line = r_d_r.readLine();
			String r_a_line = r_a_r.readLine();
			
			boolean contin = true;
			int lineTaken = -1;
			while(contin){
				lineTaken = getSmallest(f_d_line,f_a_line,r_d_line,r_a_line);
				if(lineTaken==0){
					writer.write(f_d_line+"\n");
					f_d_line = f_d_r.readLine();
				}
				else if(lineTaken==1){
					writer.write(f_a_line+"\n");
					f_a_line = f_a_r.readLine();
				}
				else if(lineTaken==2){
					writer.write(r_d_line+"\n");
					r_d_line = r_d_r.readLine();
				}
				else if(lineTaken==3){
					writer.write(r_a_line+"\n");
					r_a_line = r_a_r.readLine();
				}
				
				contin = !(f_d_line==null && f_a_line==null && r_d_line==null && r_a_line==null);
			}
			
			writer.flush();
			f_d_r.close();
			f_a_r.close();
			r_d_r.close();
			r_a_r.close();
			writer.close();
			
			f_d.delete();
			f_a.delete();
			r_d.delete();
			r_a.delete();
		}
		catch(Exception exc){
			System.err.println(exc);
		}
	}
	
	//returns which one of the parameters contains the smallest, see the TODO of
	//mergeAndDeleteResults for a better make-up.
	private static int getSmallest(String line_0,String line_1,String line_2, String line_3){
		int _0 = Integer.MAX_VALUE; 	
		int _1 = Integer.MAX_VALUE;
		int _2 = Integer.MAX_VALUE;
		int _3 = Integer.MAX_VALUE;
		
		if(line_0!=null) _0 = Integer.parseInt(line_0.split(" ")[0]);
		if(line_1!=null) _1 = Integer.parseInt(line_1.split(" ")[0]);
		if(line_2!=null) _2 = Integer.parseInt(line_2.split(" ")[0]);
		if(line_3!=null) _3 = Integer.parseInt(line_3.split(" ")[0]);
		
		int[] array = {_0,_1,_2,_3};
		int minimum = _0;
		int index = 0;
		for(int i= 0;i<array.length;i++){
			if(array[i]<minimum){
				minimum = array[i];
				index= i;
			}
		}
		return index;
	}
	
	
	/**
	 * This method makes an identical copy of a file (new filename is old filename
	 * with _copy attached at the end), and then deletes the original. This method
	 * is necessary when applying the feature selection filters on a line-by-line method
	 * rather than a file-by-file method. 
	 * @param original The original file
	 * @return The copy of the original file, with as filename _copy attached
	 */
	public static File makeCopy_DestroyOriginal(File original){
		File copy = null;
		try{
			BufferedReader reader = new BufferedReader(new FileReader(original));
			copy = new File(original.getPath()+"_copy");
			BufferedWriter writer = new BufferedWriter(new FileWriter(copy));
			
			String s = reader.readLine();
			while(s!=null){
				writer.write(s+"\n");
				s = reader.readLine();
			}
			reader.close();
			writer.close();
			original.delete();
		}
		catch(Exception exc){
			System.out.println(exc);
		}
		return copy;
	}
	
	
	/**
	 * This method generates the Gnuplot-commands to display the roc-curves 
	 * (only possible with cross-validation). Two different gnuplot-command
	 * files are generated : 
	 * a) The first one displays all the roc-curves on a single display. While
	 * this works for a limited amount of roc-curves that are also rather divergent,
	 * it has limited applicability when the amount of roc-curves increases.
	 * b) A command which displays (and stores as png-files) all couples of roc-curves.
	 * This is thus easily usable for comparing all the roc-curves.
	 * @param fileNames A list with all the filenames and data which contain roc-curve data
	 * @param path The path in which to place the resulting gnuplot-command file
	 * @param isDonor Indicates whether we are dealing with donors or acceptors 
	 */
	public static void writeRocGnuPlotCommand (List<RocCurveData> fileNames,String path,boolean isDonor){
		try{
			File f = null;
			if(isDonor)
				f = new File(path+"roc_don.sh");
			else
				f = new File(path+"roc_acc.sh");
			BufferedWriter writer = new BufferedWriter(new FileWriter(f));
			writer.write("plot ");
			for(int i=0;i<fileNames.size();i++){
				if(i!=fileNames.size()-1)
					writer.write("'"+fileNames.get(i).getFileName()+"' with lines, ");
				else
					writer.write("'"+fileNames.get(i).getFileName()+"' with lines");
			}
			writer.write("\n");
			writer.flush();
			writer.close();
			
			
			File fd = null;
			if(isDonor)
				fd = new File(path+"roc_don_dual.sh");
			else
				fd = new File(path+"roc_acc_dual.sh");
			BufferedWriter wr = new BufferedWriter(new FileWriter(fd));
			for(int i=0;i<fileNames.size();i++){
				for(int j=0;j<fileNames.size();j++){
					wr.write("set terminal png\n");
					if(isDonor)
						wr.write("set output 'roc_output_donor_"+fileNames.get(i).getIdentifier()+"_"+fileNames.get(j).getIdentifier()+".png' \n");
					else
						wr.write("set output 'roc_output_acceptor_"+fileNames.get(i).getIdentifier()+"_"+fileNames.get(j).getIdentifier()+".png' \n");
					
					wr.write("set grid \n");
					wr.write("set xlabel \"FP-Rate\" \n");
					wr.write("set ylabel \"TP-Rate\" \n");
					
					wr.write("plot '"+fileNames.get(i).getFileName()+"' with lines, '"+fileNames.get(j).getFileName()+"' with lines \n");
					wr.write("\n");
				}
			}
			wr.flush();
			wr.close();
			
		}
		catch(Exception exc){
			System.out.println(exc);
		}
	}
	
	
	/**
	 * Returns the number of lines (terminated by a carriage return) in the file
	 * @param file The file of which we want the number of lines
	 * @return The number of lines
	 */
	public static int getLinesInFile(File file){
		int result = 0;
		try{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			while(s!=null){
				result++;
				s = reader.readLine();
			}
			reader.close();
		}
		catch(Exception exc){
			System.out.println(exc);
		}
		return result;
	}
	
	
	/**
	 * Returns the extension of a filename
	 * @param fileName The name of the file
	 * @return The extension of the file
	 */
	public static String getExtension(String fileName){
		StringBuffer sb = new StringBuffer();
		for(int i=fileName.length()-1;i>=0;i--){
			if(fileName.charAt(i)!='.')
				sb.append(fileName.charAt(i));
			else{
				sb.append('.');
				return reverseString(new String(sb));
			}
		}
		return "";
	}
	
	/**
	 * Reverses a string (last character becomes the first and vice versa)
	 * @param s The string to be reversed
	 * @return The new reversed string
	 */
	private static String reverseString(String s){
		StringBuffer res = new StringBuffer();
		for(int i=s.length()-1;i>=0;i--)
			res.append(s.charAt(i));
		return new String(res);
	}
	
	
	
	/**
	 * This methods writes the results of a crossvalidation to a file. This includes 
	 * pretty basic stuff such as the amount of true positives and false positives, but also
	 * deducted information such as the F_1 measure and the specificity. 
	 * 
	 * While it may seem incoherent to need both the featurefile and trainPos/trainNeg data, 
	 * it is clear this is needed because we may use a special case of crossvalidation in 
	 * which the data is not split evenly, thus we need more information about the total
	 * amount of trainingdata
	 * 
	 * It is assumed that the crossvalidationresult has allready computed all the deducted
	 * information.
	 * 
	 * @param result The crossvalidation result
	 * @param options The different Jaspr options
	 * @param logger The logging facility (used for error-output)
	 * @param isDonor indicates whether we are dealing with donor or acceptor data 
	 * @param featureFile The file containing the training-features (used for determining the total amount of positives and negatives)
	 * @param trainPos The number positive training examples used
	 * @param testPos The number of positive test examples used 
	 * @param trainNeg The number of negative training examples used
	 * @param testNeg The number of negative test examples used
	 */
	public static void writeCrossValidation(CrossValidationResult result,Options options,Logger logger, boolean isDonor,File featureFile
			,int trainPos, int testPos, int trainNeg, int testNeg){
		try{
			File output = null;
			if(isDonor)
				output = new File(options.OUTPUTDIRECTORY+"donor_crossvalidation.txt");
			else
				output = new File(options.OUTPUTDIRECTORY+"acceptor_crossvalidation.txt");

			BufferedWriter writer = new BufferedWriter(new FileWriter(output,true));
			writer.write("TOTAL_POSITIVES : "+result.getTotalPositives()+"\t| TRUE_POSITIVES : "+result.getTruePositives()+"\t| FALSE_NEGATIVES : "+result.getFalseNegatives()+"\n");
			writer.write("TOTAL_NEGATIVES : "+result.getTotalNegatives()+"\t| TRUE_NEGATIVES : "+result.getTrueNegatives()+"\t| FALSE_POSITIVES : "+result.getFalsePositives()+"\n");
			writer.write("ERROR_RATE: "+result.getError_rate()+"\n");
			writer.write("ACCURACY: "+result.getAccuracy2()+"\n");
			writer.write("SENSITIVITY :"+result.getSensitivity()+"\n");
			writer.write("FP_RATE :"+result.getFp_rate()+"\n");
			writer.write("SPECIFICITY :"+result.getSpecificity()+"\n");
			writer.write("CORRELATION_COEFFICIENT :"+result.getCorrelation_coefficient()+"\n");
			writer.write("APPROXIMATE_CORRELATION :"+result.getApproximate_correlation()+"\n");
			writer.write("F1_MEASURE :"+result.getF1_measure()+"\n");
			writer.write("FP5%_MEASURE:"+result.getFp5_measure()+"\n");
			writer.write("AUC :"+result.getAuc()+"\n");
			writer.write("====================\n");
			writer.write("CLASSIFICATION FEATURES: \n");
			List<ClassificationFeature> list = options.cfList;
			for(int i=0;i<list.size();i++){
				ClassificationFeature feat = list.get(i);
				writer.write("\t "+feat.toString()+"\n");
			}
			writer.write("====================\n");
			writer.write("DATA :\n");
			NtrFileSupport nfs = new NtrFileSupport(options,logger);
			int[]pos_neg = nfs.getPosNegNumberInFile(featureFile);
			writer.write("\t TRAINING_EXAMPLES : "+getLinesInFile(featureFile)+"\t TOTAL_POSITIVE : "+pos_neg[0]+" \t TOTAL_NEGATIVE : "+pos_neg[1]+"\n");
			writer.write("\t POSITIVE_TRAINING_EXAMPLES : "+trainPos+"\n");
			writer.write("\t NEGATIVE_TRAINING_EXAMPLES : "+trainNeg+"\n");
			writer.write("\t POSITIVE_TEST_EXAMPLES : "+testPos+"\n");
			writer.write("\t NEGATIVE_TEST_EXAMPLES : "+testNeg+"\n");
			writer.write("====================\n");
			writer.write("====================\n");
			writer.write("\n\n\n");
			writer.close();	
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	
	/**
	 * Creates and fills the temporary file with data for crossvalidation.
	 * @param currentFold The current subset (starting from 1)
	 * @param posLength The length of the subset for the positive file 
	 * @param negLength The length of the subset for the negative file
	 * @param posFile The positive file
	 * @param negFile The negative file
	 * @param isDonor Indicates whether this concerns a donor svm (just for naming purposes)
	 * @return The file with data 
	 * @throws Exception
	 */
	public static File fillCrossValTrainingFile(int currentFold,int posLength,int negLength, File posFile,File negFile,boolean isDonor){
		File file = null;
		try{
			String extension = getExtension(posFile.getName());
			if(isDonor)
				file = new File("tempCrossValDon"+extension);
			else
				file = new File("tempCrossValAcc"+extension); 
			BufferedReader posReader = new BufferedReader(new FileReader(posFile));
			BufferedReader negReader = new BufferedReader(new FileReader(negFile));
			BufferedWriter temWriter = new BufferedWriter(new FileWriter(file));
		
		//	position lines in the files
			for(int i=0;i<(currentFold-1)*posLength;i++)
				posReader.readLine();
			for(int i=0;i<(currentFold-1)*negLength;i++)
				negReader.readLine();
			
			//write the data to the file
			for(int i=(currentFold-1)*posLength;i<currentFold*posLength;i++){
				String s =  posReader.readLine();
				temWriter.write(s+"\n");
			}
			for(int i=(currentFold-1)*negLength;i<currentFold*negLength;i++){
				String s = negReader.readLine();
				temWriter.write(s+"\n");
			}
		
			posReader.close();
			negReader.close();
			temWriter.flush();
			temWriter.close();
		}
		catch(Exception exc){
			System.err.println(exc);
		}
		return file;		
	}
	
	public static CrossValidationData fillCrossValFiles(int posLength,int negLength,File posFile,File negFile, boolean isDonor){
		CrossValidationData data = new CrossValidationData();
		try{
			File training = null;
			File test = null;
			
			String extension = getExtension(posFile.getName());
			if(isDonor){
				training = new File("tempCrossTrainingDon"+extension);
				test = new File("tempCrossTestDon"+extension);
			}
			else{
				training = new File("tempCrossTrainingAcc"+extension);
				test = new File("tempCrossTestAcc"+extension);
			}
				
			Random random = new Random();
			int counter = 0;
			
			SortedSet<Integer> posSet= new TreeSet<Integer>();
			SortedSet<Integer> negSet= new TreeSet<Integer>();
			
			while(counter<posLength){
				int line = random.nextInt(posLength);
				if(!posSet.contains(line)){
					posSet.add(line);
					counter++;
				}
			}
			counter = 0;
			while(counter<negLength){
				int line = random.nextInt(negLength);
				if(!negSet.contains(line)){
					negSet.add(line);
					counter++;
				}
			}
			data = seperateData(posSet,negSet,posFile,negFile,training,test);
		}
		catch(Exception exc){
			System.err.println(exc);
		}
		return data;
	}
	
	public static CrossValidationData seperateData(SortedSet<Integer> randomPosTest, SortedSet<Integer> randomNegTest, File posFile, File negFile, File training, File test) throws Exception{
		BufferedReader posReader = new BufferedReader(new FileReader(posFile));
		BufferedReader negReader = new BufferedReader(new FileReader(negFile));
		BufferedWriter trainWriter = new BufferedWriter(new FileWriter(training));
		BufferedWriter testWriter = new BufferedWriter(new FileWriter(test));
		
		String s = posReader.readLine();
		int counter = 0;
		while(s!=null){
			if(randomPosTest.contains(counter)){
				testWriter.write(s+"\n");
			}
			else{
				trainWriter.write(s+"\n");
			}
			s = posReader.readLine();
			counter++;
		}
		
		s = negReader.readLine();
		counter = 0;
		while(s!=null){
			if(randomNegTest.contains(counter)){
				testWriter.write(s+"\n");
			}
			else{
				trainWriter.write(s+"\n");
			}
			s = posReader.readLine();
			counter++;
		}
		posReader.close();
		negReader.close();
		trainWriter.flush();
		testWriter.flush();
		trainWriter.close();
		testWriter.close();
		
		CrossValidationData data = new CrossValidationData();
		data.setTestFile(test);
		data.setTrainingFile(training);
		return data;
	
	}
}
