/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.io;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.util.List;

import org.apache.log4j.Logger;

import util.Options;
import classifier.Classifier;

public class NtrFileSupport {

	private Options options = null;
	private Logger logger = null;
	

	
	public NtrFileSupport(Options options, Logger logger){
		this.options = options;
		this.logger  = logger;
	}
	
	public NtrFileSupport(Logger logger){
		this.logger = logger;
	}
	
	public File mergeNtrData(File positiveFile, File negativeFile, boolean isDonor){
		if(options == null)
			return null;
		String destFile = null;
		File result = null;
		if(isDonor){
			destFile = options.outputFileFeaturesDonor;
		}
		else
			destFile = options.outputFileFeaturesAcceptor;
		
		if(!destFile.contains(options.OUTPUTDIRECTORY))
			destFile = options.OUTPUTDIRECTORY + destFile;
		
		try{
			result = new File(destFile);
			BufferedWriter writer = new BufferedWriter(new FileWriter(result));
			BufferedReader posReader = new BufferedReader(new FileReader(positiveFile));
			BufferedReader negReader = new BufferedReader(new FileReader(negativeFile));
			
			String s = posReader.readLine();
			while(s!=null){
				writer.write(s+"\n");
				s = posReader.readLine();
			}
			s = negReader.readLine();
			while(s!=null){
				writer.write(s+"\n");
				s = negReader.readLine();
			}
			
			posReader.close();
			negReader.close();
			writer.close();
			positiveFile.delete();
			negativeFile.delete();
		}
		catch(Exception exc){
			logger.error(exc);
		}
		return result;
	}
	
	public void mergeNtrData(File positiveFile, File negativeFile,File destiny){
		File result = destiny;
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(result));
			BufferedReader posReader = new BufferedReader(new FileReader(positiveFile));
			BufferedReader negReader = new BufferedReader(new FileReader(negativeFile));
			
			String s = posReader.readLine();
			while(s!=null){
				writer.write(s+"\n");
				s = posReader.readLine();
			}
			s = negReader.readLine();
			while(s!=null){
				writer.write(s+"\n");
				s = negReader.readLine();
			}
			
			posReader.close();
			negReader.close();
			writer.close();
			positiveFile.delete();
			negativeFile.delete();
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	
	public int[] getPosNegNumberInFile(File file){
		int pos = 0;
		int neg = 0;
		try{
			BufferedReader reader = new BufferedReader(new FileReader(file));
			String s = reader.readLine();
			while(s!=null){
				if(s.startsWith("1"))
					pos++;
				else
					neg++;
				s = reader.readLine();
			}
		}
		catch(Exception exc){
			logger.error(exc);
		}
		int[] result = {pos,neg};
		return result;
	}
	
	
	
	
	public void writeNtrData(File file, List<List<Double>> data,Classifier.DATA_TYPE dataType){
		try{
			BufferedWriter writer = new BufferedWriter(new FileWriter(file,true));
			int counter = 0;
			int lastWritten = 0;
			StringBuffer buffer = new StringBuffer();
			
			//class information
			switch(dataType){
			case POSITIVE_DATA: buffer.append("1 ");break;
			case NEGATIVE_DATA: buffer.append("-1 ");break;
			case UNCLASSIFIED_DATA : buffer.append("0 ");break;
			}
			double temp = 0;
			for(int i=0;i<data.size();i++){
				List<Double> dat = data.get(i);
				for(int j=0;j<dat.size();j++){
					counter ++;
					temp = dat.get(j);
					if(temp!=0.0){
						buffer.append(counter+":"+temp+" ");
						lastWritten = counter;
					}
				}
			}
			if(lastWritten!=counter)
				buffer.append(counter+":"+temp);
			buffer.append("\n");
			writer.write(new String(buffer));
			writer.flush();
			writer.close();
		}
		catch(Exception exc){
			logger.error(exc);
		}
	}
	

	
}
