/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.parsers.embl;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class EmblUtils {

	
	public static void main(String [] args){
		List<EmblFileData> list = parseDirectory(args[0]);
		for(int i=0;i<list.size();i++){
		/*	EmblFileData dat = list.get(i);
			List<Embl_CDS_Data> sdList = dat.getCds_data_list();
			for(int j=0;j<sdList.size();j++){
				System.out.println(sdList.get(j).getName());
			}
			System.out.println();*/
		}
	}
	
	public static List<EmblFileData>parseDirectory(String dirName){
		List<EmblFileData> list = new ArrayList<EmblFileData>();
		File dir = new File(dirName);
		if(!dir.exists() || !dir.isDirectory())
			return list;
		File []files = dir.listFiles();
		for(int i=0;i<files.length;i++){
			EmblFileData data = parseFile(files[i]);
			list.add(data);
		}
		return list;
	}
	
	public static EmblFileData parseFile(File file){
		EmblFileData data = null;
		try{
			if(!file.exists())
				return data;
			data = new EmblFileData();
			
			parseHeader(data,file);
			extractData(data);
		//	testData(data);
			
		}
		catch(Exception exc){
			System.err.println("Error while parsing "+file.getName());
			System.err.println(exc);
			exc.printStackTrace();
		}
		
		return data;
		
	}
	
	public static void testData(EmblFileData data) throws IOException{
		int testLength  = data.getSequence().length();
		int bpLength	= data.getDefinedBasePairs();
		
		System.out.println("Lengths of file "+data.getFileName()+" : " +testLength+" <-> "+bpLength);
	}
	
	private static void extractData(EmblFileData data) throws IOException{
		List<Embl_CDS_Data> list = data.getCds_data_list();
		String sequence = data.getSequence();
		for(int i=0;i<list.size();i++){
			Embl_CDS_Data cds_data = list.get(i);
			List<Integer> ss = cds_data.getSplicesites();
			
			//set the amount of exons
			cds_data.setNumExons(ss.size()/2);
			
			//set the coding sequence length
			int cds_length = 0;
			for(int j=1;j<ss.size();j+=2){
				int temp = ss.get(j)-ss.get(j-1);
				cds_length+=temp;
			}
			cds_data.setCds_length(cds_length);
			if(cds_length%3==0)
				cds_data.setCorrectCDSLength(true);
			else
				cds_data.setCorrectCDSLength(false);
			
			//set the start- and stopcodons
			boolean reverse = cds_data.isReverse();
			String startc = "";
			String stopc  = "";
			boolean start_c = true;
			boolean stop_c  = true;
			if(reverse){
				//start
				int loc = ss.get(ss.size()-1);
				startc = sequence.substring(loc-3,loc);
				start_c = startc.equals("CAT");
				//stop 
				loc = ss.get(0);
				stopc = sequence.substring(loc-1,loc+2);
				stop_c = stopc.equals("CTA")||stopc.equals("TCA")||stopc.equals("TTA");
				
			}
			else{
				//start
				int loc = ss.get(0);
				startc = sequence.substring(loc-1,loc+2);
				start_c = startc.equals("ATG");
				//stop
				loc = ss.get(ss.size()-1);
				stopc = sequence.substring(loc-3,loc);
				stop_c = stopc.equals("TGA")||stopc.equals("TAG")||stopc.equals("TAA");
				
			}
			cds_data.setStartCodon(startc);
			cds_data.setCorrectStartCodon(start_c);
			cds_data.setStopCodon(stopc);
			cds_data.setCorrectStopCodon(stop_c);
			
			//extract the boundaries
			List<String> boundaries = new ArrayList<String>();
			if(reverse){
				for(int j=1;j<ss.size()-1;j++){
					String dinuc = "";
					int splice = ss.get(j);
					if(j%2==0){
						dinuc = sequence.substring(splice-3,splice-1);
					}
					else{
						dinuc = sequence.substring(splice,splice+2);
					}
					boundaries.add(dinuc);
				}

			}
			else{
				for(int j=1;j<ss.size()-1;j++){
					String dinuc = "";
					int splice = ss.get(j);
					if(j%2==0){
						dinuc = sequence.substring(splice-3,splice-1);
					}
					else{
						dinuc = sequence.substring(splice,splice+2);
					}
					boundaries.add(dinuc);
				}
			}
			cds_data.setBoundaries(boundaries);
		}
	}
	
	private static void parseHeader(EmblFileData data, File file) throws IOException{
		BufferedReader reader = new BufferedReader(new FileReader(file));
		String s = reader.readLine();
		List<Embl_CDS_Data> cds_data_list = new ArrayList<Embl_CDS_Data>();
		int basePairs = 0;
		String sequence = "";
		
		while(s!=null){
			if(s.startsWith("FT")){
				if(s.indexOf("CDS")!=-1){
					Embl_CDS_Data cds_data = new Embl_CDS_Data();
					List<Integer> res = new ArrayList<Integer>();
					if(s.indexOf("complement")==-1)
						cds_data.setReverse(false);
					else
						cds_data.setReverse(true);
					
					int firstIndex = s.indexOf("join")+5;
					if(s.indexOf("join")==-1&&s.indexOf("complement")!=-1)
						firstIndex = s.indexOf("(")+1;
					if(s.indexOf("join")==-1&&s.indexOf("complement")==-1){
						firstIndex = s.length();
						String[] split = s.split(" ");
						String fin = split[split.length-1];
						int temp = tryIntegerParse(fin.substring(0,fin.indexOf(".")));
						res.add(temp);
						temp = tryIntegerParse(fin.substring(fin.lastIndexOf(".")+1,fin.length()));
						res.add(temp);
					}
					
					int temp = 0;
					String t = "";
					for(int i=firstIndex;i<s.length();i++){
						char c = s.charAt(i);
						
						if(c==')'){
							temp = tryIntegerParse(t);
							res.add(temp);
							break;
						}
						else if(c==','){
							temp = tryIntegerParse(t);
							res.add(temp);
							t = "";
						}
						else if(c=='.'){
							temp = tryIntegerParse(t);
							res.add(temp);
							t = "";
							i++;
						}
						else 
							t+=c;
					}
					cds_data.setSplicesites(res);
					cds_data.setName(reader.readLine());
					cds_data_list.add(cds_data);
				}
			}
			else if(s.startsWith("SQ")){
				String sub = s.substring(s.indexOf("Sequence")+9,s.indexOf("BP")-1);
				basePairs = Integer.parseInt(sub);
			}
			else{
				s = s.trim();
				String [] split = s.split(" ");
				for(int i=0;i<split.length;i++){
					if(split[i].length()==10)
						sequence+=split[i];
					else{
						String temp = evalSeqString(split[i]);
						sequence+=temp;
					}
				}
			}
			
			s = reader.readLine();
		}
		
		data.setDefinedBasePairs(basePairs);
		data.setSequence(sequence);
		data.setCds_data_list(cds_data_list);
		data.setFileName(file.getName());
	}
	
	private static int tryIntegerParse(String s){
		int start = 0;
		int stop = s.length();
		if(s.startsWith(">"))
			start = 1;
		if(s.startsWith("<"))
			start = 1;
		
		return Integer.parseInt(s.substring(start,stop));
			
	}
	
	
	private static String evalSeqString(String s){
		if(s==null||s.equals(""))
			return "";
		String result = "";
		String temp = s.toUpperCase();
		StringBuffer res = new StringBuffer();
		for(int i=0;i<temp.length();i++){
			char c = temp.charAt(i);
			if(c=='A'||c=='C'||c=='G'||c=='T'||c=='U')
				res.append(s.charAt(i));
		}
		if(res.length()!=0)
			result = new String(res);
		
		return result;
	}
	
}
