/*
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package util.results;

/**
 * Class that computes the alfa and beta values for a sigmoid function.
 * Code is taken from original SpliceMachine code.
 * @author Michiel Van Bel,Sven Degroeve, Yvan Saeys
 */

/*
 * Original code :
 * Hieronder de code van trainSigmoid om de sigmoid functie te bepalen.
De functie wordt opgeroepen als
trainSigmoid (predictions, target, neg, pos, &A, &B);
waarbij predictions een array is van de (double) distances, target een array
van de klassen (-1 en +1) en neg en pos het aantal negatieve en positieve
instances in de training set is. Parameters A en B worden door de functie
ingevuld.

succes,

yvan


inline double
Log (double number)
{
  if (number == 0.0)
    {
      return -200;
    }
  else
    {
      return (log (number));
    }
}

inline double
Log2 (double number)
{
  if (number == 0.0)
    {
      return -200;
    }
  else
    {
      return (log (number) / log (2));
    }
}


void
trainSigmoid (double *out, double *y, int prior0, int prior1, double *A,
          double *B)
{
//      double hiTarget = (double)(prior0+1)/(double)(prior1+2);
//      double loTarget = 1 / (prior0+2);
  double hiTarget = (1 - (1 / (prior1 + 2)));    // ADJUSTED FOR 
UNBALANEED DATA
  double loTarget = (1 / (prior0 + 2));    // ADJUSTED FOR UNBALAMCED DATA
  double lambda = 1e-3;
  double olderr = 1e300;
  int l = (prior0 + prior1);
  double *pp = (double *) malloc (sizeof (double) * l);
  int count = 0;
  double oldA;
  double oldB;
  double err;
  double det;
  double p;
  double diff;
  double scale;
  double t;
  int u;
  int it;
  int i;
  *A = 0.0;
  *B = log ((double) (prior0 + 1) / (double) (prior1 + 2));
  for (u = 0; u < l; u++)
    {
      pp[u] = (double) (prior1 + 1) / (double) (prior0 + prior1 + 2);
    }
  for (it = 1; it <= 100; it++)
    {
      double a = 0.0;
      double b = 0.0;
      double c = 0.0;
      double d = 0.0;
      double e = 0.0;
      for (i = 0; i < l; i++)
    {
      double d1, d2;
      if (y[i] > 0)
        {
          t = hiTarget;
        }
      else
        {
          t = loTarget;
        }
      d1 = pp[i] - t;
      d2 = pp[i] * (1 - pp[i]);
      a += out[i] * out[i] * d2;
      b += d2;
      c += out[i] * d2;
      d += out[i] * d1;
      e += d1;
    }
      if ((fabs (d) < 1e-9) && (fabs (e) < 1e-9))
    {
      break;
    }
      oldA = *A;
      oldB = *B;
      err = 0.0;
      while (1)
    {
      det = (a + lambda) * (b + lambda) - (c * c);
      if (det == 0)
        {
          lambda *= 10;
          continue;
        }
      *A = oldA + ((b + lambda) * d - c * e) / det;
      *B = oldB + ((a + lambda) * e - c * d) / det;
      err = 0.0;
      for (i = 0; i < l; i++)
        {
          if (y[i] > 0)
        {
          t = hiTarget;
        }
          else
        {
          t = loTarget;
        }
          p = 1 / (1 + exp (out[i] * (*A) + (*B)));
          pp[i] = p;
          err -= t * Log (p) + (1 - t) * Log (1 - p);
        }
      if (err < olderr * (1 + 1e-7))
        {
          lambda *= 0.1;
          break;
        }
      lambda *= 10;
      if (lambda >= 1e6)
        {
          break;
        }
    }
      diff = err - olderr;
      scale = 0.5 * (err + olderr + 1);
      if ((diff > -1e-3 * scale) && (diff < 1e-7 * scale))
    {
      count++;
    }
      else
    {
      count = 0;
    }
      olderr = err;
      if (count == 3)
    {
      break;
    }
    }
  free (pp);
}
 */

public class SigmoidFunction {

	public static void main(String [] args){
		//double[] out = {-1,-2,-3,-1,-2,-3,-1,-2,-3,-1,-2,-3,-1,-2,-3,-4,-2,-3,-2,-1,1,2};
		//double[] y   = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,1,1};
		double[] out = {-1,-2,1,2};
		double[] y   = {-1,-1,1,1};
		int numneg = 2;
		int numpos = 2;
		Double[] res = new Double[2];
		SigmoidFunction sf = new SigmoidFunction();
		sf.trainSigmoid(out,y,numneg,numpos,res);
		
		System.out.println("Sigmoid : a = "+res[0]+" | b = "+res[1]);
	}
	
	public SigmoidFunction(){
	}
	
	/**
	 * Computes the sigmoid function for data, and stores the resulting A and B values
	 * in a passed Parameter. Coding style is C-like (but it is just C-code revamped
	 * as Java code).
	 * 
	 * @param out The distances to the SVM hyperplane from the predictions
	 * @param y An array which contains all classes (in most cases only 2 : +1 /-1)
	 * @param numNeg The number of negative training instances
	 * @param numPos The number of positive training instances
	 * @param A_B The resulting array which contains at the first position the A value, and at
	 * the second position it contains the B value.
	 */
	public void trainSigmoid(double[] out, double [] y, int numNeg, int numPos, Double[]A_B){
		
	/*	for(int i=0;i<out.length;i++){
			System.out.println("training distance : "+i+" -> "+out[i]);
		}*/
		
		double hiTarget	= (1-(1/(numPos+2)));	//adjusted for unbalanced data
		double loTarget	= (1/(numNeg+2));		//adjusted for unbalanced data
		double lambda 	= 1e-3;
		double olderr	= 1e300;
		int l 			= numNeg+numPos;
		double[] pp		= new double[l];
		int count		= 0;
		double oldA		= 0;
		double oldB		= 0;
		double err		= 0;
		double det		= 0;
		double p		= 0;
		double diff		= 0;
		double scale	= 0;
		double t		= 0;
		int u			= 0;
		int it			= 0;
		int i			= 0;
		
		A_B[0] 			= 0.0;	//pass by ref doesnt work, so we just put A and B in an array
		A_B[1]			= Math.log((double)(numNeg+1)/(double)(numNeg+2));
		for(u=0;u<l;u++){
			pp[u] = (double)(numPos+1) / (double)(numNeg+numPos+2);
		}
		for(it=1;it<=100;it++){
			double a = 0.0;
		    double b = 0.0;
		    double c = 0.0;
		    double d = 0.0;
		    double e = 0.0;
		    for(i=0;i<l;i++){
		    	double d1 	= 0.0;
		    	double d2	= 0.0;
		    	if(y[i]>0){
		    		t = hiTarget;
		    	}
		    	else{
		    		t = loTarget;
		    	}
		    	d1 = pp[i]-t;
		    	d2 = pp[i] * (1 - pp[i]);
		        a += out[i] * out[i] * d2;
		        b += d2;
		        c += out[i] * d2;
		        d += out[i] * d1;
		        e += d1;
		    }
		    if((Math.abs(d)<1e-9) && (Math.abs(e)<1e-9)){
		    	break;
		    }
		    oldA = A_B[0];
		    oldB = A_B[1];
		    err	 = 0.0;
		    while(true){
		    	det+=(a+lambda)*(b+lambda)-(c*c);
		    	if(det==0.0 || det==0){			//originally this was det==0, but det is double??
		    		lambda *=10;
		    		continue;
		    	}
		    	A_B[0] = oldA + ((b + lambda) * d - c * e) / det;
		    	A_B[1] = oldB + ((a + lambda) * e - c * d) / det;
		    	err = 0.0;
		        for (i = 0; i < l; i++){
		        	if (y[i] > 0){
		              t = hiTarget;
		            }
		        	else{
		              t = loTarget;
		            }
		        	p = 1 / (1 + Math.exp (out[i] * (A_B[0]) + (A_B[1])));
		        	pp[i] = p;
		            err -= t * Log (p) + (1 - t) * Log (1 - p);
		        }
		        if (err < olderr * (1 + 1e-7)){
		        	lambda *= 0.1;
		        	break;
		        }
		        lambda *= 10;
		        if (lambda >= 1e6){
		        	break;
		        }
		    }
		    diff = err-olderr;
		    scale = 0.5 * (err + olderr + 1);
		    if ((diff > -1e-3 * scale) && (diff < 1e-7 * scale)){
		    	count++;
		    }
		    else{
		    	count = 0;
		    }
		    olderr = err;
		    if (count == 3){
		    	break;
		    }
		}
		//System.out.println("#of iterations for sigmoid : "+it);
	}
	
	public double Log(double number){
		if(number==0.0)
			return -200;
		else
			return Math.log(number);
	}
	
	
	public double Log2(double number){
		if(number == 0.0)
			return -200;
		else
			return (Math.log(number)/Math.log(2));
	}
	
}
