package var;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import javax.swing.BorderFactory;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileNameExtensionFilter;

import var.experimenter.ActionPanel;
import var.experimenter.ConfigurationFileReader;
import var.experimenter.RangeOption;
import var.experimenter.RangePanel;

public class Experimenter extends JPanel implements ActionListener{

	public final static long serialVersionUID = 44587l;
	public final static String VERSION = "v0.7.9";
	private String configName = "./var/config/config2.txt";
	
	private JMenuItem newAction = null;
	private JMenuItem removeAction = null;
	private JMenuItem saveItem = null;
	private JMenuItem loadItem = null;
	private JMenuItem exitItem = null;
	private JMenuItem helpItem = null;
	private JMenuItem aboutItem = null;
	
	private JCheckBox autoDirectory = null;
	private JCheckBox greedyScripts = null;
	private JCheckBox groupScriptsBox = null;
	
	private JRadioButton unixButton = null;
	private JRadioButton win32Button = null;
	
	private JTextField mappingField	 = null;
	private JButton exitButton = null;
	private JButton createButton = null;
	
	private RangePanel rangePanel = null;
//	private JScrollPane scroll = null;
	
	public static void main(final String[] args){
		SwingUtilities.invokeLater(new Runnable() {
            public void run() {
            	new Experimenter(args);
            }
		});
	}
	
	
	public Experimenter(String [] args){
		super();
		parseArguments(args);
		Dimension size = new Dimension(1200,700);
		JFrame frame = new JFrame("JASPr experimenter "+VERSION);
		this.setMinimumSize(size);
		this.setMaximumSize(size);
		this.setPreferredSize(size);
		
		createContent();
		
		JMenuBar menuBar = createMenuBar();
		frame.setJMenuBar(menuBar);
		JFrame.setDefaultLookAndFeelDecorated(true);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setContentPane(this);
		frame.setVisible(true);
		frame.setResizable(false);
		frame.pack();
	}
	
	
	public void actionPerformed(ActionEvent e){
		if(e.getSource().equals(exitButton)||e.getSource().equals(exitItem)){
			System.exit(0);
		}
		else if(e.getSource().equals(helpItem)){
			JOptionPane.showMessageDialog(this, "Please consult ./config/config.txt for help");
		}
		else if(e.getSource().equals(aboutItem)){
			String message = "FunSiP Experimenter "+VERSION+"\n\n"+
			"The FunSiP Experimenter needs a full version\n of FunSiP for the generated scripts " +
			"to be functional. \n\n"+
			"(c) Michiel Van Bel\n     VIB Flanders\n     PSB UGent";
			JOptionPane.showMessageDialog(this, message);
		}
		else if(e.getSource().equals(saveItem)){
			JFileChooser chooser = new JFileChooser();
		    FileNameExtensionFilter filter = new FileNameExtensionFilter(
		            "FunSiP Experimenter Files", "sef");
		    chooser.setFileFilter(filter);
		    int returnVal = chooser.showSaveDialog(this);
		    if(returnVal==JFileChooser.APPROVE_OPTION){
		    	File file = chooser.getSelectedFile();
		    	if(!file.getName().endsWith(".sef"))
		    		file = new File(file.getAbsolutePath()+".sef");
		    	try{
		    		List<ActionPanel> list = rangePanel.getAllActions();
		    		FileOutputStream stream = new FileOutputStream(file);
		    		ZipOutputStream zos = new ZipOutputStream(stream);
		    		zos.putNextEntry(new ZipEntry(file.getName()));
		    		ObjectOutput s = new ObjectOutputStream(zos);
		    		s.writeObject(list);
		    		zos.closeEntry();
		    		stream.close();
		    		JOptionPane.showMessageDialog(this,"Experiment saved");
		    	}
		    	catch(Exception exc){
		    		//exc.printStackTrace();
		    		JOptionPane.showMessageDialog(this, "Error while saving:\n"+exc.toString(),"Cannot save",JOptionPane.ERROR_MESSAGE);
		    	}
		    }
		}
		else if(e.getSource().equals(loadItem)){
			JFileChooser chooser = new JFileChooser();
		    FileNameExtensionFilter filter = new FileNameExtensionFilter(
		            "FunSiP Experimenter Files", "sef");
		    chooser.setFileFilter(filter);
		    int returnVal = chooser.showOpenDialog(this);
		    if(returnVal==JFileChooser.APPROVE_OPTION){
		    	File file = chooser.getSelectedFile();
		    	try{
		    		FileInputStream stream = new FileInputStream(file);
		    		ZipInputStream zis = new ZipInputStream(stream);
		    		zis.getNextEntry();
		    		ObjectInput s = new ObjectInputStream(zis);
		    		List<? extends ActionPanel> tempList = (List<? extends ActionPanel>)s.readObject();
		    		s.close();
		    		List<ActionPanel> data = new ArrayList<ActionPanel>();
		    		if(tempList!=null){
		    			data.addAll(tempList);
		    			rangePanel.clearAllActions();
		    			rangePanel.setActions(data);
		    		//	scroll.revalidate();
			    		JOptionPane.showMessageDialog(this, "Experiment loaded");
		    		}
		    		else{
		    			JOptionPane.showMessageDialog(this,"No data found in save-file","Unable to load experiment",JOptionPane.ERROR_MESSAGE);
		    		}	
		    	}
		    	catch(Exception exc){
		    		exc.printStackTrace();
		    		JOptionPane.showMessageDialog(this, "Error while loading:\n"+exc.toString(),"Cannot load",JOptionPane.ERROR_MESSAGE);
		    	}
		    }
			
			
		}
		else if(e.getSource().equals(createButton)){
			//boolean unix = unixButton.isSelected();
			//boolean groupScripts  = groupScriptsBox.isSelected();
			if(greedyScripts.isSelected()){}
			else{}
			JOptionPane.showMessageDialog(this,"Scripts generated", "Scripts generated", JOptionPane.INFORMATION_MESSAGE);
				
		}
		else if(e.getSource().equals(newAction)){
			createActionPanel();
		}
		else if(e.getSource().equals(removeAction)){
			removeActionPanel();
		}
	}
	
	
	private void createActionPanel(){
		ActionPanel actionPanel = new ActionPanel();
		String name = JOptionPane.showInputDialog(this,"Enter a name for this action");
		List<RangeOption> list = ConfigurationFileReader.readConfigurationFile(configName);
		actionPanel.setName(name);
		actionPanel.setRangeOptions(list);
		rangePanel.addActionPanel(actionPanel);
		
		this.repaint();
		this.revalidate();
	}
	
	private void removeActionPanel(){
		int index = rangePanel.getCurrentActionPanelIndex();
		rangePanel.removeActionPanel(index);
	}
	
	private void setSizes(Dimension d, JPanel p){
		p.setPreferredSize(d);
		p.setMaximumSize(d);
		p.setMinimumSize(d);
	}
	
	private void createContent(){
		Dimension southPanelDimension 		= new Dimension(1200,200);
		Dimension buttonPanelDimension 		= new Dimension(1200,70);
		Dimension optionsPanelDimension 	= new Dimension(1200,130);
		Dimension rangePanelDimension 		= new Dimension(1200,500);
		
		rangePanel = new RangePanel(rangePanelDimension);
		JPanel southPanel = new JPanel();
		JPanel buttonPanel = new JPanel();
		JPanel optionsPanel = new JPanel();
		
		setSizes(southPanelDimension,southPanel);
		setSizes(buttonPanelDimension,buttonPanel);
		setSizes(optionsPanelDimension,optionsPanel);
		
		rangePanel.setBorder(BorderFactory.createEtchedBorder());
		southPanel.setBorder(BorderFactory.createEtchedBorder());
		buttonPanel.setBorder(BorderFactory.createEtchedBorder());
		optionsPanel.setBorder(BorderFactory.createTitledBorder("Options"));
		
		optionsPanel.setLayout(new GridLayout(3,3));
		autoDirectory			= new JCheckBox("Auto create directories",true);
		JLabel mappingLabel 	= new JLabel("Name mappingfile");
		mappingField			= new JTextField("Mapping.txt",15);
		greedyScripts  			= new JCheckBox("Greedy optimization(recommanded)",true);
		unixButton 				= new JRadioButton("Unix",true);
		win32Button 			= new JRadioButton("Win32",false);
		ButtonGroup group 		= new ButtonGroup();
		group.add(unixButton);
		group.add(win32Button);
		groupScriptsBox 		= new JCheckBox("Group scripts and directories",true);
		optionsPanel.add(autoDirectory);
		optionsPanel.add(mappingLabel);
		optionsPanel.add(mappingField);
		optionsPanel.add(greedyScripts);
		optionsPanel.add(unixButton);
		optionsPanel.add(win32Button);
		optionsPanel.add(groupScriptsBox);
		
		
		exitButton 			= new JButton("Exit");
		createButton 		= new JButton("Create scripts");
		exitButton.addActionListener(this);
		createButton.addActionListener(this);
		buttonPanel.add(createButton);
		buttonPanel.add(exitButton);
		
		southPanel.setLayout(new BorderLayout());
		southPanel.add(optionsPanel,BorderLayout.NORTH);
		southPanel.add(buttonPanel,BorderLayout.CENTER);
		this.setLayout(new BorderLayout());
		
		setSizes(rangePanelDimension,rangePanel);
		this.add(rangePanel,BorderLayout.NORTH);
		this.add(southPanel,BorderLayout.SOUTH);
	}
	
	private JMenuBar createMenuBar(){
		JMenuBar menubar = new JMenuBar();
		
		JMenu fileMenu = new JMenu("File");
		fileMenu.setMnemonic('F');
		newAction = new JMenuItem("Add new action",KeyEvent.VK_N);
		newAction.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N,ActionEvent.CTRL_MASK));
		removeAction = new JMenuItem("Remove action",KeyEvent.VK_R);
		saveItem = new JMenuItem("Save current experiment",KeyEvent.VK_S);
		saveItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,ActionEvent.CTRL_MASK));
		loadItem = new JMenuItem("Load experiment",KeyEvent.VK_L);
		loadItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,ActionEvent.CTRL_MASK));
		exitItem = new JMenuItem("Exit",KeyEvent.VK_X);
		newAction.addActionListener(this);
		removeAction.addActionListener(this);
		saveItem.addActionListener(this);
		loadItem.addActionListener(this);
		exitItem.addActionListener(this);
		fileMenu.add(newAction);
		fileMenu.add(removeAction);
		fileMenu.addSeparator();
		fileMenu.add(saveItem);
		fileMenu.add(loadItem);
		fileMenu.addSeparator();
		fileMenu.add(exitItem);
		
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');
		helpItem = new JMenuItem("Help",KeyEvent.VK_H);
		aboutItem = new JMenuItem("About",KeyEvent.VK_A);
		helpItem.addActionListener(this);
		aboutItem.addActionListener(this);
		helpMenu.add(helpItem);
		helpMenu.addSeparator();
		helpMenu.add(aboutItem);
		
		menubar.add(fileMenu);
		menubar.add(helpMenu);
		return menubar;
	}
	
	
	private void parseArguments(String[] args){
		for(int i=0;i<args.length;i++){
			if(args[i].equals("-configfile")){
				configName = args[++i];
			}
			else{
				System.err.println("unrecognized option : "+args[i]);
			}
		}
	}
}
